"""

"""
import logging

from ..servers.afc import AFCClient, AFCShell
from ..util.lockdown import LockdownClient


class HouseArrestService(AFCClient):
    SERVICE_NAME = "com.apple.mobile.house_arrest"

    def __init__(self, lockdown=None, udid=None, network=None,logger=None):
        self.logger = logger or logging.getLogger(__name__)
        self.lockdown = lockdown or LockdownClient(udid=udid,network=network)
        super(HouseArrestService, self).__init__(self.lockdown, self.SERVICE_NAME)

    def stop_session(self):
        self.logger.info("Disconecting...")
        self.service.close()

    def send_command(self, applicationId, cmd="VendContainer"):
        self.service.send_plist({"Command": cmd, "Identifier": applicationId})
        res = self.service.recv_plist()
        if res.get("Error"):
            self.logger.error("%s : %s", applicationId, res.get("Error"))
            return False
        else:
            return True

    def shell(self, applicationId, cmd="VendContainer"):
        res = self.send_command(applicationId, cmd)
        if res:
            AFCShell(client=self).cmdloop()


if __name__ == '__main__':
    HouseArrestService().shell('cn.rongcloud.rce.autotest.xctrunner')



import json
import os
import plistlib
import threading
import uuid
from distutils.version import LooseVersion

from ios_device.servers.Installation import InstallationProxyService
from ios_device.servers.Instrument import InstrumentServer
from ios_device.servers.dvt import DTXEnum
from ios_device.servers.house_arrest import HouseArrestService
from ios_device.servers.testmanagerd import TestManagerdLockdown
from ios_device.util import Log
from ios_device.util.bpylist2 import NSUUID, NSURL, XCTestConfiguration
from ios_device.util.bpylist2 import archive
from ios_device.util.dtx_msg import RawObj, RawInt32sl, RawInt64sl
from ios_device.util.exceptions import InstrumentRPCParseError
from ios_device.util.gpu_decode import JSEvn, TraceData
from ios_device.util.kperf_data import KperfData
from ios_device.util.lockdown import LockdownClient
from ios_device.util.variables import InstrumentsService, LOG

log = Log.getLogger(LOG.Instrument.value)


class InstrumentDeviceInfo:
    def __exit__(self):
        self.rpc.stop()

    def __init__(self, rpc: InstrumentServer):
        self.rpc = rpc

    def runningProcesses(self):
        """ 获取当前运行应用
        :return:
        """
        parsed = self.rpc.call(InstrumentsService.DeviceInfo, "runningProcesses").selector
        return parsed

    def execnameForPid(self, pid):
        """ 获取应用路径
        :param pid:
        :return:
        """
        parsed = self.rpc.call(InstrumentsService.DeviceInfo, "execnameForPid:", str(pid)).selector
        return parsed

    def isRunningPid(self, pid):
        """ 应用是否运行
        :param pid:
        :return:
        """
        parsed = self.rpc.call(InstrumentsService.DeviceInfo, "isRunningPid:", str(pid)).selector
        return parsed

    def nameForUID(self, uid):
        parsed = self.rpc.call(InstrumentsService.DeviceInfo, "nameForUID:", str(uid)).selector
        return parsed

    def machTimeInfo(self):
        """ 时间校准，获取
        :return: mach time 比例
        """
        parsed = self.rpc.call(InstrumentsService.DeviceInfo, "machTimeInfo").selector
        return parsed

    def traceCodesFile(self):
        """ traceCodes 堆栈 code 码
        :return:
        """
        parsed = self.rpc.call(InstrumentsService.DeviceInfo, "traceCodesFile").selector
        return {int(k, 16): v for k, v in map(lambda l: l.split(), parsed.splitlines())}

    def networkInformation(self):
        """ 当前网络信息
        :return:
        """
        parsed = self.rpc.call(InstrumentsService.DeviceInfo, "networkInformation").selector
        return parsed

    def systemInformation(self):
        """ 设备基本信息
        :return:
        """
        parsed = self.rpc.call(InstrumentsService.DeviceInfo, "systemInformation").selector
        return parsed

    def hardwareInformation(self):
        """ 硬件数据
        :return:
        """
        parsed = self.rpc.call(InstrumentsService.DeviceInfo, "hardwareInformation").selector
        return parsed

    def sysmonProcessAttributes(self):
        """ 获取应用性能数据所需的参数
        :return:
        """
        parsed = self.rpc.call(InstrumentsService.DeviceInfo, "sysmonProcessAttributes").selector
        return parsed

    def sysmonSystemAttributes(self):
        """ 获取系统性能数据所需的参数
        :return:
        """
        parsed = self.rpc.call(InstrumentsService.DeviceInfo, "sysmonSystemAttributes").selector
        return parsed

    def symbolicatorSignaturesForExpiredPids(self):
        parsed = self.rpc.call(InstrumentsService.DeviceInfo,
                               "symbolicatorSignaturesForExpiredPids").selector
        return parsed

    def directoryListingForPath(self, path: str):
        """ 监听文件夹路径
        :param path:
        :return:
        """
        parsed = self.rpc.call(InstrumentsService.DeviceInfo, "directoryListingForPath:",
                               path).selector
        return parsed

    def iconDescriptionFileForAppPath(self, path: str):
        """ 获取应用 icon 数据
        :param path: /private/var/containers/Bundle/Application/A86066DC-C145-4900-90E7-6FB0CF72215A/smoba.app
        :return:
        """

        parsed = self.rpc.call(InstrumentsService.DeviceInfo, "iconDescriptionFileForAppPath:",
                               path).selector
        return parsed

    def kpepDatabase(self):
        parsed = self.rpc.call(InstrumentsService.DeviceInfo, "kpepDatabase").selector
        return plistlib.loads(parsed)

    def machKernelName(self):
        parsed = self.rpc.call(InstrumentsService.DeviceInfo, "cpKDebugEventsAsXML").selector
        return parsed


class InstrumentsBase:
    def __init__(self, udid=None, network=None, lockdown=None):
        self.udid = udid
        self.network = network
        self.instruments_rcp = None
        self.manager_rpc = None
        self.lockdown = lockdown or LockdownClient(udid=udid, network=network)
        self.process_attributes = None
        self.system_attributes = None

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        if self.instruments_rcp:
            self.instruments_rcp.stop()
        if self.manager_rpc:
            self.manager_rpc.stop()

    def get_pid(self, bundle_id=None, name=None):
        pid = None
        if bundle_id:
            app = self.application_listing(bundle_id)
            if app:
                name = app.get('ExecutableName')

        if name:
            processes = self.device_info.runningProcesses()
            for p in processes:
                if name == p.get('name'):
                    pid = p.get('pid')
        return pid

    @property
    def instruments(self):
        if self.instruments_rcp:
            return self.instruments_rcp
        self.instruments_rcp = InstrumentServer(lockdown=self.lockdown).init()
        return self.instruments_rcp

    @property
    def manager_lockdown(self):
        if self.manager_rpc:
            return self.manager_rpc
        self.manager_rpc = TestManagerdLockdown(lockdown=self.lockdown).init()
        return self.manager_rpc

    @property
    def device_info(self) -> InstrumentDeviceInfo:
        rpc = self.instruments
        return InstrumentDeviceInfo(rpc)

    def launch_app(self, bundle_id,
                   app_env=None,
                   app_args: list = [],
                   app_path: str = "",
                   options: dict = {},
                   callback: callable = None):

        if app_env is None:
            app_env = {}
        else:
            app_env = json.loads(app_env)
            if not isinstance(app_env, dict):
                log.info('app_env 参数异常应为 Json 格式')
                return

        options = options or {
            "StartSuspendedKey": 0,
            "KillExisting": False,
        }
        if callback:
            self.instruments.register_channel_callback(InstrumentsService.ProcessControl, callable)
        pid = self.instruments.call(InstrumentsService.ProcessControl,
                                    "launchSuspendedProcessWithDevicePath:bundleIdentifier:environment:arguments"
                                    ":options:",
                                    app_path, bundle_id, app_env, app_args, options).selector
        return pid

    def kill_app(self, pid):
        parsed = self.instruments.call(InstrumentsService.ProcessControl, "killPid:", str(pid)).selector
        return parsed

    def signal_app(self, sig, pid):
        parsed = self.instruments.call(InstrumentsService.ProcessControl, "sendSignal:toPid", str(sig),
                                       str(pid)).selector
        return parsed

    def application_listing(self, bundle_id=None):
        """ _selector
            - installedApplicationsMatching:registerUpdateToken:
            - unregisterUpdateToken:
        :return:
        """
        applist = self.instruments.call(InstrumentsService.ApplicationListing,
                                        "installedApplicationsMatching:registerUpdateToken:",
                                        {}, "").selector
        # ret = applist
        if bundle_id:
            for app in applist:
                if app.get('CFBundleIdentifier') == bundle_id:
                    return app
        # return ret

    def sysmontap(self,
                  callback: callable,
                  time: int = 1000,
                  stopSignal: threading.Event = threading.Event()):
        """
        :param time: 获取时间间隔（ms）
        :param stopSignal:
        :param callback:
        :return:
        """
        self.process_attributes = self.process_attributes or list(self.device_info.sysmonProcessAttributes())
        self.system_attributes = self.system_attributes or list(self.device_info.sysmonSystemAttributes())

        log.info(f'Sysmontap setConfig ...')
        self.instruments.call(InstrumentsService.Sysmontap, "setConfig:", {
            'ur': time,  # 输出频率 ms
            'bm': 0,
            'procAttrs': self.process_attributes,  # 输出所有进程信息字段，字段顺序与自定义相同（全量自字段，按需使用）
            'sysAttrs': self.system_attributes,  # 系统信息字段
            'cpuUsage': True,
            'sampleInterval': time * 1000000})
        self.instruments.register_channel_callback(InstrumentsService.Sysmontap, callback)
        self.instruments.call(InstrumentsService.Sysmontap, "start")
        log.info(f'Sysmontap start ...')
        log.info(f'wait for data ...')
        while not stopSignal.wait(1):
            pass
        self.instruments.call(InstrumentsService.Sysmontap, "stop")

    def graphics(self,
                 callback: callable,
                 time=1000,
                 stopSignal: threading.Event = threading.Event()):
        """ 获取 FPS 等数据
        :param time: 获取时间间隔（ms）
        :param callback:
        :param stopSignal:
        :return:
        """
        self.instruments.register_channel_callback(InstrumentsService.GraphicsOpengl, callback)
        log.info(self.instruments.call(InstrumentsService.GraphicsOpengl, "availableStatistics").selector)
        log.info(self.instruments.call(InstrumentsService.GraphicsOpengl, "driverNames").selector)
        self.instruments.call(InstrumentsService.GraphicsOpengl, "setSamplingRate:",
                              float(time / 100))
        self.instruments.call(InstrumentsService.GraphicsOpengl,
                              "startSamplingAtTimeInterval:",
                              0.0)
        while not stopSignal.wait(1):
            pass
        self.instruments.call(InstrumentsService.GraphicsOpengl, "stopSampling")

    def xcode_network(self, pid, stopSignal: threading.Event = threading.Event()):
        channel = "com.apple.xcode.debug-gauge-data-providers.NetworkStatistics"
        attr = {}
        self.instruments.call(channel, "startSamplingForPIDs:", {pid})
        while not stopSignal.wait(1):
            ret = self.instruments.call(channel, "sampleAttributes:forPIDs:", attr, {pid})
            print(ret.selector)

    def networking(self,
                   callback: callable,
                   stopSignal: threading.Event = threading.Event()):
        """ 获取网络数据
        :param callback:
        :param stopSignal:
        :return:
        """
        self.instruments.register_channel_callback(InstrumentsService.Networking, callback)

        self.instruments.call(InstrumentsService.Networking, "replayLastRecordedSession")
        log.info(msg=f'networking replayLastRecordedSession')
        self.instruments.call(InstrumentsService.Networking, "startMonitoring")
        log.info(msg=f'networking startMonitoring networking')
        self.instruments.register_selector_callback(DTXEnum.FINISHED, lambda _: stopSignal.set())
        while not stopSignal.wait(1):
            pass
        self.instruments.call(InstrumentsService.Networking, "stopMonitoring")
        log.info(msg=f'networking stopMonitoring ')

    def mobile_notifications(self,
                             callback: callable,
                             stopSignal: threading.Event = threading.Event()):

        self.instruments.register_undefined_callback(callback)
        self.instruments.call(InstrumentsService.MobileNotifications,
                              'setApplicationStateNotificationsEnabled:', str(True))
        log.info(msg='Waiting for action app')
        while not stopSignal.wait(1):
            pass
        self.instruments.call(InstrumentsService.MobileNotifications,
                              'setApplicationStateNotificationsEnabled:', str(False))

    def core_profile_session(self,
                             callback: callable,
                             stopSignal: threading.Event = threading.Event()):
        self.instruments.register_channel_callback(InstrumentsService.CoreProfileSessionTap,
                                                   callback)
        self.instruments.call(InstrumentsService.CoreProfileSessionTap, "setConfig:",
                              {'rp': 10,
                               'tc': [{'kdf2': {630784000, 833617920, 830472456},
                                       'tk': 3,
                                       'uuid': str(uuid.uuid4()).upper()}],
                               'ur': 500})
        self.instruments.call(InstrumentsService.CoreProfileSessionTap, "start")
        while not stopSignal.wait(1):
            pass
        self.instruments.call(InstrumentsService.CoreProfileSessionTap, "stop")

    def gpu_counters(self,
                     callback: callable,
                     stopSignal: threading.Event = threading.Event()):
        self.instruments.register_undefined_callback(callback)
        requestDeviceGPUInfo = self.instruments.call(InstrumentsService.GPU, 'requestDeviceGPUInfo').selector

        min_collection_interval = requestDeviceGPUInfo[0].get('min-collection-interval')
        self.instruments.call(InstrumentsService.GPU,
                              "configureCounters:counterProfile:interval:windowLimit:tracingPID:",
                              RawInt64sl(min_collection_interval, 3, 1, 0), RawInt32sl(-1))
        self.instruments.call(InstrumentsService.GPU, 'startCollectingCounters')
        log.info('Wait for gup counters data ...')
        while not stopSignal.wait(1):
            pass
        self.instruments.call(InstrumentsService.GPU, 'stopCollectingCounters')
        data = self.instruments.call(InstrumentsService.GPU, 'flushRemainingData').selector
        return data

    def screenshot(self):
        var = self.instruments.call(InstrumentsService.Screenshot, "takeScreenshot").selector
        return var

    def power(self,
              callback: callable,
              stopSignal: threading.Event = threading.Event()):
        channel = "com.apple.instruments.server.services.power"
        self.instruments.register_channel_callback(channel, callback)
        stream_num = self.instruments.call(channel, "openStreamForPath:", "live/level.dat").selector
        print("open", stream_num)
        print("start", self.instruments.call(channel, "startStreamTransfer:", float(stream_num)).selector)
        print("[!] wait a few seconds, be patient...")
        while not stopSignal.wait(1):
            pass
        log.info(f"stop{self.instruments.call(channel, 'endStreamTransfer:', float(stream_num)).selector}")

    def xcode_energy(self, pid, stopSignal: threading.Event = threading.Event()):
        self.instruments.call(InstrumentsService.XcodeEnergy, "startSamplingForPIDs:", {pid})
        while not stopSignal.wait(1):
            ret = self.instruments.call(InstrumentsService.XcodeEnergy, "sampleAttributes:forPIDs:", {}, {pid})
            print(ret.selector)

    def get_condition_inducer(self):
        """ 获取网络配置参数，用于 condition_inducer
        :return:
        """
        ret = self.instruments.call(InstrumentsService.ConditionInducer, "availableConditionInducers").selector
        return ret

    def set_condition_inducer(self,
                              condition_identifier,
                              profile_identifier,
                              stopSignal: threading.Event = threading.Event()):
        """ 设置手机状态，模拟网络，手机压力数据等
        :param condition_identifier:
        :param profile_identifier:
        :return:
        """
        ret = self.instruments.call(InstrumentsService.ConditionInducer,
                                    'enableConditionWithIdentifier:profileIdentifier:',
                                    condition_identifier, profile_identifier).selector
        log.info(
            f"Condition configuration enabled [{profile_identifier}] successfully, please do not stop the command...")

        while not stopSignal.wait(1):
            pass
        return ret

    def disable_condition_inducer(self):
        """ 关闭手机状态，模拟网络，手机压力数据等
        """
        c = self.instruments.call(InstrumentsService.ConditionInducer, 'disableActiveCondition').selector
        return c

    def xctest(self, bundle_id, USE_PORT='', quit_event=threading.Event()):
        log = Log.getLogger(LOG.xctest.value)

        def _callback(res):
            log.info(f" {res.selector} : {res.auxiliaries}")

        with InstallationProxyService(lockdown=self.lockdown) as install:
            app_info = install.find_bundle_id(bundle_id)
            if not app_info:
                log.warning(f"No app matches {bundle_id}")
                return

        log.info("BundleID: %s", bundle_id)
        log.info("DeviceIdentifier: %s", self.lockdown.device_info.get('UniqueDeviceID'))
        sign_identity = app_info.get("SignerIdentity", "")
        log.info("SignIdentity: %r", sign_identity)
        XCODE_VERSION = 29
        session_identifier = NSUUID(bytes=os.urandom(16), version=4)
        ManagerdLockdown1 = TestManagerdLockdown(self.lockdown).init()

        ManagerdLockdown1.make_channel("dtxproxy:XCTestManager_IDEInterface:XCTestManager_DaemonConnectionInterface")
        if self.lockdown.ios_version > LooseVersion('11.0'):
            result = ManagerdLockdown1.call(
                "dtxproxy:XCTestManager_IDEInterface:XCTestManager_DaemonConnectionInterface",
                "_IDE_initiateControlSessionWithProtocolVersion:", RawObj(XCODE_VERSION)).selector
            log.info("result: %s", result)
        ManagerdLockdown1.register_selector_callback(DTXEnum.FINISHED, lambda _: quit_event.set())
        ManagerdLockdown1.register_undefined_callback(_callback)

        ManagerdLockdown2 = TestManagerdLockdown(self.lockdown).init()
        ManagerdLockdown2.make_channel("dtxproxy:XCTestManager_IDEInterface:XCTestManager_DaemonConnectionInterface")
        ManagerdLockdown2.register_selector_callback(DTXEnum.FINISHED, lambda _: quit_event.set())
        ManagerdLockdown2.register_undefined_callback(_callback)

        _start_flag = threading.Event()

        def _start_executing(res=None):
            if _start_flag.is_set():
                return
            _start_flag.set()

            log.info("Start execute test plan with IDE version: %d",
                     XCODE_VERSION)
            ManagerdLockdown2._call(False, 0xFFFFFFFF, '_IDE_startExecutingTestPlanWithProtocolVersion:',
                                    RawObj(XCODE_VERSION))

        def _show_log_message(res):
            log.info(f"{res.selector} : {res.auxiliaries}")
            if 'Received test runner ready reply with error: (null' in ''.join(
                    res.auxiliaries):
                log.info("Test runner ready detected")
                _start_executing()

        ManagerdLockdown2.register_selector_callback('_XCT_testBundleReadyWithProtocolVersion:minimumVersion:',
                                                     _start_executing)
        ManagerdLockdown2.register_selector_callback('_XCT_logDebugMessage:', _show_log_message)
        ManagerdLockdown2.register_selector_callback('_XCT_didFinishExecutingTestPlan', lambda _: quit_event.set())

        result = ManagerdLockdown2.call('dtxproxy:XCTestManager_IDEInterface:XCTestManager_DaemonConnectionInterface',
                                        '_IDE_initiateSessionWithIdentifier:forClient:atPath:protocolVersion:',
                                        RawObj(
                                            session_identifier,
                                            str(session_identifier) + '-6722-000247F15966B083',
                                            '/Applications/Xcode.app/Contents/Developer/usr/bin/xcodebuild',
                                            XCODE_VERSION
                                        )).selector
        log.info("result: %s", result)
        # launch_wda
        xctest_path = "/tmp/WebDriverAgentRunner-" + str(session_identifier).upper() + ".xctestconfiguration"
        xctest_content = archive(XCTestConfiguration({
            "testBundleURL": NSURL(None, "file://" + app_info['Path'] + "/PlugIns/WebDriverAgentRunner.xctest"),
            "sessionIdentifier": session_identifier,
        }))

        fsync = HouseArrestService(self.lockdown)
        fsync.send_command(bundle_id)
        for fname in fsync.read_directory("/tmp"):
            if fname.endswith(".xctestconfiguration"):
                log.debug("remove /tmp/%s", fname)
                fsync.file_remove("/tmp/" + fname)
        fsync.set_file_contents(xctest_path, xctest_content)

        conn = InstrumentServer(self.lockdown).init()
        conn.call('com.apple.instruments.server.services.processcontrol', 'processIdentifierForBundleIdentifier:',
                  bundle_id)

        conn.register_undefined_callback(_callback)
        app_path = app_info['Path']
        app_container = app_info['Container']

        xctestconfiguration_path = app_container + xctest_path
        log.info("AppPath: %s", app_path)
        log.info("AppContainer: %s", app_container)

        app_env = {
            'CA_ASSERT_MAIN_THREAD_TRANSACTIONS': '0',
            'CA_DEBUG_TRANSACTIONS': '0',
            'DYLD_FRAMEWORK_PATH': app_path + '/Frameworks:',
            'DYLD_LIBRARY_PATH': app_path + '/Frameworks',
            'NSUnbufferedIO': 'YES',
            'SQLITE_ENABLE_THREAD_ASSERTIONS': '1',
            'WDA_PRODUCT_BUNDLE_IDENTIFIER': '',
            'XCTestConfigurationFilePath': xctestconfiguration_path,
            'XCODE_DBG_XPC_EXCLUSIONS': 'com.apple.dt.xctestSymbolicator',
            'MJPEG_SERVER_PORT': '',
            'USE_PORT': str(USE_PORT),
        }
        if self.lockdown.ios_version > LooseVersion('11.0'):
            app_env['DYLD_INSERT_LIBRARIES'] = '/Developer/usr/lib/libMainThreadChecker.dylib'
            app_env['OS_ACTIVITY_DT_MODE'] = 'YES'
        app_options = {'StartSuspendedKey': False}
        if self.lockdown.ios_version > LooseVersion('12.0'):
            app_options['ActivateSuspended'] = True

        app_args = [
            '-NSTreatUnknownArgumentsAsOpen', 'NO',
            '-ApplePersistenceIgnoreState', 'YES'
        ]

        identifier = "launchSuspendedProcessWithDevicePath:bundleIdentifier:environment:arguments:options:"

        pid = conn.call('com.apple.instruments.server.services.processcontrol', identifier,
                        app_path, bundle_id, app_env, app_args, app_options).selector
        if not isinstance(pid, int):
            log.error(f"Launch failed: {pid}")
            raise Exception("Launch failed")

        log.info(f"Launch {bundle_id} pid: {pid}")

        conn.call('com.apple.instruments.server.services.processcontrol', "startObservingPid:", RawObj(pid))

        if quit_event:
            conn.register_selector_callback(DTXEnum.FINISHED, lambda _: quit_event.set())

        if self.lockdown.ios_version > LooseVersion('12.0'):
            identifier = '_IDE_authorizeTestSessionWithProcessID:'
            result = ManagerdLockdown1.call(
                'dtxproxy:XCTestManager_IDEInterface:XCTestManager_DaemonConnectionInterface',
                identifier,
                RawObj(pid)).selector
            log.info("_IDE_authorizeTestSessionWithProcessID: %s", result)
        else:
            identifier = '_IDE_initiateControlSessionForTestProcessID:protocolVersion:'
            result = ManagerdLockdown1.call(
                'dtxproxy:XCTestManager_IDEInterface:XCTestManager_DaemonConnectionInterface',
                identifier,
                RawObj(pid), RawObj(XCODE_VERSION)).selector
            log.info("_IDE_authorizeTestSessionWithProcessID: %s", result)

        while not quit_event.wait(.1):
            pass
        log.warning("xctrunner quited")
        conn.stop()
        ManagerdLockdown2.stop()
        ManagerdLockdown1.stop()

    def core_profile(self, config, pid, name, stopSignal: threading.Event = threading.Event()):
        def on_graphics_message(res):
            if type(res.selector) is InstrumentRPCParseError:
                for args in Kperf.to_str(res.selector.data):
                    print(args)

        self.instruments.register_channel_callback("com.apple.instruments.server.services.coreprofilesessiontap",
                                                   on_graphics_message)
        traceCodesFile = self.device_info.traceCodesFile()
        Kperf = KperfData(traceCodesFile, pid, name)
        self.instruments.call("com.apple.instruments.server.services.coreprofilesessiontap", "setConfig:", config)
        self.instruments.call("com.apple.instruments.server.services.coreprofilesessiontap", "start")
        while not stopSignal.wait(1):
            pass
        self.instruments.call("com.apple.instruments.server.services.coreprofilesessiontap", "stop")
        self.instruments.stop()



"""
@Date    : 2021-01-28
@Author  : liyachao
"""
import uuid
from datetime import datetime
from numpy import long, mean

from ios_device.util.exceptions import InstrumentRPCParseError
from ios_device.servers.Installation import InstallationProxyService
from ios_device.servers.Instrument import InstrumentServer
from ios_device.util import api_util
from ios_device.util.api_util import PyIOSDeviceException, RunXCUITest
from ios_device.util.forward import ForwardPorts
from ios_device.util.kperf_data import KperfData
from ios_device.util.utils import kperf_data


class PyiOSDevice:
    def __init__(self, device_id: str = None, rpc_channel: InstrumentServer = None):
        self.device_id = device_id
        self.xcuitest = None
        self.rpc_channel = None
        if not rpc_channel or not rpc_channel._cli:
            self.init()
        else:
            self.rpc_channel = rpc_channel

    def init(self):
        if not self.rpc_channel:
            self.rpc_channel = init(self.device_id)

    def get_processes(self):
        return get_processes(self.device_id, self.rpc_channel)

    def stop(self):
        self.rpc_channel.stop()

    def get_channel(self):
        """
        获取当前设备有哪些服务
        :return:
        """

        return self.rpc_channel._published_capabilities

    def start_get_gpu(self, callback: callable):
        """
        开始获取 gpu 数据
        :param callback:
        :return:
        """
        return start_get_gpu(device_id=self.device_id, rpc_channel=self.rpc_channel, callback=callback)

    def stop_get_gpu(self):
        """
        结束获取 gpu 数据
        :return:
        """
        stop_get_gpu(self.rpc_channel)

    def launch_app(self, bundle_id: str = None):
        """
        启动 app
        :param bundle_id:
        :return:
        """
        return launch_app(device_id=self.device_id, rpc_channel=self.rpc_channel, bundle_id=bundle_id)

    def start_get_network(self, callback: callable):
        """
        开始获取上下行流量
        :param callback:
        :return:
        """
        return start_get_network(device_id=self.device_id, rpc_channel=self.rpc_channel, callback=callback)

    def stop_get_network(self):
        """
        结束获取网络包内容
        :return:
        """
        stop_get_network(rpc_channel=self.rpc_channel)

    def start_get_system(self, callback: callable):
        """
        开始获取系统数据
        :param callback:
        :return:
        """
        return start_get_system(device_id=self.device_id, rpc_channel=self.rpc_channel, callback=callback)

    def stop_get_system(self):
        """
        结束获取系统数据
        :return:
        """
        stop_get_system(rpc_channel=self.rpc_channel)

    def get_device(self):
        """
        获取设备对象用于操作设备
        :return:
        """
        return get_device(device_id=self.device_id, rpc_channel=self.rpc_channel)

    def get_applications(self):
        """
        获取手机应用列表
        :return:
        """
        return get_applications(device_id=self.device_id, rpc_channel=self.rpc_channel)

    def start_xcuitest(self, bundle_id, callback: callable, app_env: dict = None,
                       pair_ports=None):
        """
        启动 xcuittest
        :param pair_ports: 端口对的数组，每对端口中前一个代表远程端口，后一个代表本地端口，例如：["8100:8100", "8200:8200"]
        :param forward: 是否进行端口转发
        :param bundle_id:
        :param callback:
        :param app_env:
        :return:
        """

        self.xcuitest = start_xcuitest(bundle_id, callback, self.device_id, app_env, pair_ports)
        return self.xcuitest

    def stop_xcuitest(self, xcuitest=None):
        """
        停止 xcuitest
        :param xcuitest:
        :return:
        """
        if not xcuitest and not self.xcuitest:
            raise PyIOSDeviceException("xcuitest object can not be None")
        stop_xcuitest(xcuitest if xcuitest else self.xcuitest)

    def start_get_fps(self, callback: callable):
        """
        开始获取 fps 相关数据
        :param callback:
        :return:
        """
        return start_get_fps(device_id=self.device_id, rpc_channel=self.rpc_channel, callback=callback)

    def stop_get_fps(self):
        """
        结束获取 fps 相关数据
        :return:
        """
        stop_get_fps(rpc_channel=self.rpc_channel)

    def get_energy(self, pid: int):
        get_energy(pid=pid, device_id=self.device_id, rpc_channel=self.rpc_channel)

    def start_get_graphics_fps(self, callback: callable):
        """
        graphics 计算 fps
        :param callback:
        :return:
        """
        start_get_graphics_fps(device_id=self.device_id, rpc_channel=self.rpc_channel, callback=callback)

    def stop_get_graphics_fps(self):
        stop_get_graphics_fps(rpc_channel=self.rpc_channel)

    def start_get_mobile_notifications(self, callback: callable):
        """
        监听事件，比如 app 唤醒，杀死，退出到后台等等
        :param callback:
        :return:
        """
        start_get_mobile_notifications(device_id=self.device_id, rpc_channel=self.rpc_channel, callback=callback)

    def stop_get_mobile_notifications(self):
        stop_get_mobile_notifications(rpc_channel=self.rpc_channel)

    def get_netstat(self, pid: int):
        """
        获取单应用的网络信息
        :param pid:
        :return:
        """
        return get_netstat(pid=pid, device_id=self.device_id, rpc_channel=self.rpc_channel)

    def start_forward(self, pair_ports=None):
        """
        iOS真机设备的端口转发
        :param pair_ports: list 端口对的数组，每对端口中前一个代表远程端口，后一个代表本地端口，例如：["8100:8100", "8200:8200"]
        :param udid:
        :param threaded:
        :param bufsize:
        """
        return start_forward(pair_ports=pair_ports, device_id=self.device_id)

    def stop_forward(self, forward: ForwardPorts):
        stop_forward(forward)


def init(device_id: str = None):
    rpc_channel = InstrumentServer(udid=device_id)
    rpc_channel.init()
    return rpc_channel


def get_processes(device_id: str = None, rpc_channel: InstrumentServer = None):
    """
    获取设备的进程列表
    :param rpc_channel:
    :param device_id:
    :return:
    """
    if not rpc_channel:
        _rpc_channel = init(device_id)
    else:
        _rpc_channel = rpc_channel
    running = _rpc_channel.call("com.apple.instruments.server.services.deviceinfo", "runningProcesses").selector
    if not rpc_channel:
        _rpc_channel.stop()
    return running


def get_channel(device_id: str = None, rpc_channel: InstrumentServer = None):
    """
    当前设备可用服务列表
    :return:
    """
    if not rpc_channel:
        _rpc_channel = init(device_id)
    else:
        _rpc_channel = rpc_channel
    device_channel = _rpc_channel._published_capabilities
    if not rpc_channel:
        _rpc_channel.stop()
    return device_channel


def start_get_gpu(device_id: str = None, rpc_channel: InstrumentServer = None, callback: callable = None,
                  ms_return: bool = False):
    """

    :param device_id:
    :param rpc_channel:
    :param callback:
    :param ms_return:
    :return:
    """

    if not callback:
        raise PyIOSDeviceException("callback can not be None")

    if not rpc_channel:
        _rpc_channel = init(device_id)
    else:
        _rpc_channel = rpc_channel

    def _callback(res):
        api_util.caller(res, callback)

    if ms_return:
        _rpc_channel.call("com.apple.instruments.server.services.graphics.opengl", "setSamplingRate:", 0.0)
    _rpc_channel.call("com.apple.instruments.server.services.graphics.opengl",
                      "startSamplingAtTimeInterval:processIdentifier:",
                      0.0, 0.0)
    _rpc_channel.register_channel_callback("com.apple.instruments.server.services.graphics.opengl", _callback)

    return _rpc_channel


def stop_get_gpu(rpc_channel: InstrumentServer):
    """
    停止获取 gpu 性能数据
    :param rpc_channel:
    :return:
    """
    rpc_channel.call("com.apple.instruments.server.services.graphics.opengl", "stopSampling")


def launch_app(bundle_id: str, device_id: str = None, rpc_channel: InstrumentServer = None):
    """
    启动 app
    :param device_id:
    :param rpc_channel:
    :param bundle_id:
    :return:
    """

    if not rpc_channel:
        _rpc_channel = init(device_id)
    else:
        _rpc_channel = rpc_channel

    channel_name = "com.apple.instruments.server.services.processcontrol"
    _rpc_channel.register_channel_callback(channel_name, lambda x: x)
    pid = _rpc_channel.call(channel_name,
                            "launchSuspendedProcessWithDevicePath:bundleIdentifier:environment:arguments:options:", "",
                            bundle_id, {}, [], {"StartSuspendedKey": 0, "KillExisting": 1}).selector
    if not rpc_channel:
        _rpc_channel.stop()
    return pid


def start_get_network(callback: callable, device_id: str = None, rpc_channel: InstrumentServer = None, ):
    """
    开始获取网络包内容
    :param device_id:
    :param rpc_channel:
    :param callback:
    :return:
    """

    if not rpc_channel:
        _rpc_channel = init(device_id)
    else:
        _rpc_channel = rpc_channel

    def _callback(res):
        api_util.network_caller(res, callback)

    _rpc_channel.register_channel_callback("com.apple.instruments.server.services.networking", _callback)
    _rpc_channel.call("com.apple.instruments.server.services.networking", "replayLastRecordedSession")
    _rpc_channel.call("com.apple.instruments.server.services.networking", "startMonitoring")
    return _rpc_channel


def stop_get_network(rpc_channel: InstrumentServer):
    """
    结束获取网络包内容
    :param rpc_channel:
    :return:
    """
    rpc_channel.call("com.apple.instruments.server.services.networking", "stopMonitoring")


def start_get_system(device_id: str = None, rpc_channel: InstrumentServer = None, callback: callable = None):
    """
    开始获取系统数据
    :param device_id:
    :param rpc_channel:
    :param callback:
    :return:
    """
    if not callback:
        raise PyIOSDeviceException("callback can not be None")

    if not rpc_channel:
        _rpc_channel = init(device_id)
    else:
        _rpc_channel = rpc_channel

    def _callback(res):
        api_util.system_caller(res, callback)

    _rpc_channel.register_undefined_callback(lambda x: x)
    _rpc_channel.call("com.apple.instruments.server.services.sysmontap", "setConfig:", {
        'ur': 1000,  # 输出频率 ms
        'bm': 0,
        'procAttrs': ['memVirtualSize', 'cpuUsage', 'procStatus', 'appSleep', 'uid', 'vmPageIns', 'memRShrd',
                      'ctxSwitch', 'memCompressed', 'intWakeups', 'cpuTotalSystem', 'responsiblePID', 'physFootprint',
                      'cpuTotalUser', 'sysCallsUnix', 'memResidentSize', 'sysCallsMach', 'memPurgeable',
                      'diskBytesRead', 'machPortCount', '__suddenTerm', '__arch', 'memRPrvt', 'msgSent', 'ppid',
                      'threadCount', 'memAnon', 'diskBytesWritten', 'pgid', 'faults', 'msgRecv', '__restricted', 'pid',
                      '__sandbox'],  # 输出所有进程信息字段，字段顺序与自定义相同（全量自字段，按需使用）
        'sysAttrs': ['diskWriteOps', 'diskBytesRead', 'diskBytesWritten', 'threadCount', 'vmCompressorPageCount',
                     'vmExtPageCount', 'vmFreeCount', 'vmIntPageCount', 'vmPurgeableCount', 'netPacketsIn',
                     'vmWireCount', 'netBytesIn', 'netPacketsOut', 'diskReadOps', 'vmUsedCount', '__vmSwapUsage',
                     'netBytesOut'],  # 系统信息字段
        'cpuUsage': True,
        'sampleInterval': 1000000000})
    _rpc_channel.register_channel_callback("com.apple.instruments.server.services.sysmontap", _callback)
    _rpc_channel.call("com.apple.instruments.server.services.sysmontap", "start")
    return _rpc_channel


def stop_get_system(rpc_channel: InstrumentServer):
    """
    结束获取系统数据
    :param rpc_channel:
    :return:
    """
    if not rpc_channel:
        raise PyIOSDeviceException("rpc_channel can not be None")
    rpc_channel.call("com.apple.instruments.server.services.sysmontap", "stop")


def get_device(device_id: str = None, rpc_channel: InstrumentServer = None):
    """
    获取设备对象用于操作设备
    :param device_id:
    :param rpc_channel:
    :return:
    """
    current_device = InstallationProxyService(udid=device_id, lockdown=rpc_channel.lockdown if rpc_channel else None)
    return current_device


def get_applications(device_id: str = None, rpc_channel: InstrumentServer = None):
    """
    获取手机应用列表
    :param device_id:
    :param rpc_channel:
    :return:
    """
    if not rpc_channel:
        _rpc_channel = init(device_id)
    else:
        _rpc_channel = rpc_channel
    application_list = _rpc_channel.call(
        "com.apple.instruments.server.services.device.applictionListing",
        "installedApplicationsMatching:registerUpdateToken:",
        {}, "").selector
    if not rpc_channel:
        _rpc_channel.stop()
    return application_list


def start_xcuitest(bundle_id: str, callback: callable, device_id: str = None, app_env: dict = None,
                   pair_ports=None):
    """
    启动 xcuittest
    :param pair_ports: 端口对的数组，每对端口中前一个代表远程端口，后一个代表本地端口，例如：["8100:8100", "8200:8200"]
    :param bundle_id:
    :param callback:
    :param device_id:
    :param app_env: 启动配置 {
            'CA_ASSERT_MAIN_THREAD_TRANSACTIONS': '0',
            'CA_DEBUG_TRANSACTIONS': '0',
            'DYLD_FRAMEWORK_PATH': app_path + '/Frameworks:',
            'DYLD_LIBRARY_PATH': app_path + '/Frameworks',
            'NSUnbufferedIO': 'YES',
            'SQLITE_ENABLE_THREAD_ASSERTIONS': '1',
            'WDA_PRODUCT_BUNDLE_IDENTIFIER': '',
            'XCTestConfigurationFilePath': xctestconfiguration_path,
            'XCODE_DBG_XPC_EXCLUSIONS': 'com.apple.dt.xctestSymbolicator',
            'MJPEG_SERVER_PORT': '',
            'USE_PORT': '',
        }
    :return: 返回 xcuitest 对象,用于停止 xcuitest
    """

    if pair_ports is None:
        pair_ports = ["8100:8100"]
    xcuitest = RunXCUITest(bundle_id=bundle_id, callback=callback, device_id=device_id, app_env=app_env,
                           pair_ports=pair_ports)

    xcuitest.start()
    return xcuitest


def stop_xcuitest(xcuitest):
    """
    停止 xcuitest
    :param xcuitest: 启动时可获取对象
    :return:
    """
    if type(xcuitest) == RunXCUITest:
        xcuitest.stop()
    else:
        raise PyIOSDeviceException("参数类型必须是 RunXCUITest")


def start_get_fps(device_id: str = None, rpc_channel: InstrumentServer = None, callback: callable = None):

    """
    开始获取 fps 相关数据
    :param device_id:
    :param rpc_channel:
    :param callback:
    :return:
    """
    if not callback:
        raise PyIOSDeviceException("callback can not be None")

    if not rpc_channel:
        _rpc_channel = init(device_id)
    else:
        _rpc_channel = rpc_channel

    NANO_SECOND = 1e9  # ns
    MOVIE_FRAME_COST = 1 / 24
    last_frame = None
    last_1_frame_cost, last_2_frame_cost, last_3_frame_cost = 0, 0, 0
    jank_count = 0
    big_jank_count = 0
    jank_time_count = 0
    mach_time_factor = 125 / 3
    frame_count = 0
    time_count = 0
    count_time = datetime.now().timestamp()
    _list = []

    def _callback(res):
        nonlocal frame_count, last_frame, last_1_frame_cost, last_2_frame_cost, last_3_frame_cost, time_count, mach_time_factor, \
            jank_count, big_jank_count, jank_time_count, _list, count_time
        if type(res.selector) is InstrumentRPCParseError:
            for args in kperf_data(res.selector.data):
                _time, code = args[0], args[7]
                if code == 830472984:
                    if not last_frame:
                        last_frame = long(_time)
                    else:
                        this_frame_cost = (long(_time) - last_frame) * mach_time_factor
                        if all([last_3_frame_cost != 0, last_2_frame_cost != 0, last_1_frame_cost != 0]):
                            if this_frame_cost > mean([last_3_frame_cost, last_2_frame_cost, last_1_frame_cost]) * 2 \
                                    and this_frame_cost > MOVIE_FRAME_COST * NANO_SECOND * 2:
                                jank_count += 1
                                jank_time_count += this_frame_cost
                                if this_frame_cost > mean(
                                        [last_3_frame_cost, last_2_frame_cost, last_1_frame_cost]) * 3 \
                                        and this_frame_cost > MOVIE_FRAME_COST * NANO_SECOND * 3:
                                    big_jank_count += 1

                        last_3_frame_cost, last_2_frame_cost, last_1_frame_cost = last_2_frame_cost, last_1_frame_cost, this_frame_cost
                        time_count += this_frame_cost
                        last_frame = long(_time)
                        frame_count += 1
                # else:
                #     time_count = (datetime.now().timestamp() - count_time) * NANO_SECOND
                if time_count > NANO_SECOND:
                    callback(
                        {"currentTime": str(datetime.now()), "FPS": frame_count / time_count * NANO_SECOND,
                         "jank": jank_count,
                         "big_jank": big_jank_count, "stutter": jank_time_count / time_count})
                    jank_count = 0
                    big_jank_count = 0
                    jank_time_count = 0
                    frame_count = 0
                    time_count = 0
                    count_time = datetime.now().timestamp()

    Kperf = KperfData()
    _rpc_channel.register_undefined_callback(lambda x: x)
    # 获取mach time比例
    mach_time_info = _rpc_channel.call("com.apple.instruments.server.services.deviceinfo", "machTimeInfo").selector
    mach_time_factor = mach_time_info[1] / mach_time_info[2]
    _rpc_channel.register_channel_callback("com.apple.instruments.server.services.coreprofilesessiontap",
                                           _callback)

    _rpc_channel.call("com.apple.instruments.server.services.coreprofilesessiontap", "setConfig:",
                      {'rp': 10,
                       'tc': [{'kdf2': {630784000, 833617920, 830472456},
                               'tk': 3,
                               'uuid': str(uuid.uuid4()).upper()}],
                       'ur': 500})

    _rpc_channel.call("com.apple.instruments.server.services.coreprofilesessiontap", "start")

    return _rpc_channel


def stop_get_fps(rpc_channel: InstrumentServer):
    """
    结束获取 fps 数据
    :param rpc_channel:
    :return:
    """
    if not rpc_channel:
        raise PyIOSDeviceException("rpc_channel can not be None")
    rpc_channel.call("com.apple.instruments.server.services.coreprofilesessiontap", "stop")


def get_energy(pid: int, device_id: str = None, rpc_channel: InstrumentServer = None):
    if not rpc_channel:
        _rpc_channel = init(device_id)
    else:
        _rpc_channel = rpc_channel

    channel_name = "com.apple.xcode.debug-gauge-data-providers.Energy"
    attr = {}
    # _rpc_channel.call(channel_name, "startSamplingForPIDs:", {pid})
    ret = _rpc_channel.call(channel_name, "sampleAttributes:forPIDs:", attr, {pid})
    if not rpc_channel:
        _rpc_channel.stop()
    return ret.selector


def start_get_graphics_fps(device_id: str = None, rpc_channel: InstrumentServer = None, callback: callable = None):
    """
    graphics 计算 fps
    :param device_id:
    :param rpc_channel:
    :param callback:
    :return:
    """
    if not rpc_channel:
        _rpc_channel = init(device_id)
    else:
        _rpc_channel = rpc_channel

    def _callback(res):
        data = res.selector
        callback({"currentTime": str(datetime.now()), "fps": data['CoreAnimationFramesPerSecond']})

    _rpc_channel.register_undefined_callback(lambda x: x)
    _rpc_channel.register_channel_callback("com.apple.instruments.server.services.graphics.opengl", _callback)
    _rpc_channel.call("com.apple.instruments.server.services.graphics.opengl", "startSamplingAtTimeInterval:", 0.0)
    return _rpc_channel


def stop_get_graphics_fps(rpc_channel: InstrumentServer):
    """
    停止获取 graphics 计算 fps
    :param rpc_channel:
    :return:
    """
    if not rpc_channel:
        raise PyIOSDeviceException("rpc_channel can not be None")
    rpc_channel.call("com.apple.instruments.server.services.graphics.opengl", "stopSampling")


def start_get_mobile_notifications(device_id: str = None, rpc_channel: InstrumentServer = None,
                                   callback: callable = None):
    """
    监听事件，比如 app 唤醒，杀死，退出到后台等等
    :param device_id:
    :param rpc_channel:
    :param callback:
    :return:
    """
    if not rpc_channel:
        _rpc_channel = init(device_id)
    else:
        _rpc_channel = rpc_channel

    def _callback(res):
        callback(res)

    _rpc_channel.register_channel_callback("com.apple.instruments.server.services.mobilenotifications", _callback)

    _rpc_channel.call(
        "com.apple.instruments.server.services.mobilenotifications",
        'setApplicationStateNotificationsEnabled:', str(True))
    return _rpc_channel


def stop_get_mobile_notifications(rpc_channel: InstrumentServer):
    """

    :param rpc_channel:
    :return:
    """
    rpc_channel.call(
        "com.apple.instruments.server.services.mobilenotifications",
        'setApplicationStateNotificationsEnabled:', str(True))


def get_netstat(pid: int, device_id: str = None, rpc_channel: InstrumentServer = None):
    """
    获取单应用的网络信息
    :param device_id:
    :param rpc_channel:
    :param callback:
    :return:
    """

    if not rpc_channel:
        _rpc_channel = init(device_id)
    else:
        _rpc_channel = rpc_channel
    channel = "com.apple.xcode.debug-gauge-data-providers.NetworkStatistics"
    attr = {}
    # print("start", _rpc_channel.call(channel, "startSamplingForPIDs:", {pid}).selector)
    ret = _rpc_channel.call(channel, "sampleAttributes:forPIDs:", attr, {pid}).selector
    if not rpc_channel:
        _rpc_channel.stop()
    return ret


def start_forward(pair_ports=None, device_id: str = None):
    """
    iOS真机设备的端口转发
        :param pair_ports: list 端口对的数组，每对端口中前一个代表远程端口，后一个代表本地端口，例如：["8100:8100", "8200:8200"]
        :param device_id:
    :return:
    """
    if not pair_ports:
        pair_ports = ["8100,8100"]
    forward = ForwardPorts(pair_ports=pair_ports, device_id=device_id)
    forward.start()
    return forward


def stop_forward(forward: ForwardPorts):
    forward.stop()



"""
获取 gpu 数据，返回时间序列还没琢磨出来怎么解析

正常情况下应该是 1000 组数据返回一次，每个设备都不同，也有可能是几万返回一次
"""
import os
import sys

from ios_device.util.dtx_msg import RawInt64sl, RawInt32sl
from ios_device.util.gpu_decode import GRCDecodeOrder, GRCDisplayOrder, JSEvn, TraceData

sys.path.append(os.getcwd())

from ios_device.servers.Instrument import InstrumentServer

import time
from ios_device.util import logging

log = logging.getLogger(__name__)



def gpu(rpc):
    decode_key_list = []
    js_env:JSEvn
    display_key_list = []

    def dropped_message(res):
        nonlocal js_env,decode_key_list,display_key_list
        if res.selector[0]==1:
            js_env.dump_trace(TraceData(*res.selector))
        elif res.selector[0] == 0:
            # # print(res.selector)
            _data = res.selector[4]
            decode_key_list = GRCDecodeOrder.decode(_data.get(1))
            display_key_list = GRCDisplayOrder.decode(_data.get(0))
            js_env = JSEvn(_data.get(2), display_key_list, decode_key_list, mach_time_factor)

    rpc.register_undefined_callback(dropped_message)
    machTimeInfo = rpc.call("com.apple.instruments.server.services.deviceinfo", "machTimeInfo").selector
    mach_time_factor = machTimeInfo[1] / machTimeInfo[2]
    requestDeviceGPUInfo = rpc.call('com.apple.instruments.server.services.gpu','requestDeviceGPUInfo').selector

    min_collection_interval = requestDeviceGPUInfo[0].get('min-collection-interval')
    rpc.call("com.apple.instruments.server.services.gpu", "configureCounters:counterProfile:interval:windowLimit:tracingPID:",RawInt64sl(min_collection_interval, 3, 0,0),RawInt32sl(-1))
    rpc.call('com.apple.instruments.server.services.gpu', 'startCollectingCounters')
    time.sleep(5)

    print(rpc.call('com.apple.instruments.server.services.gpu', 'stopCollectingCounters').selector)
    data = rpc.call('com.apple.instruments.server.services.gpu', 'flushRemainingData').selector
    js_env.dump_trace(TraceData(*data[0]))
    time.sleep(2)
    rpc.stop()


if __name__ == '__main__':
    rpc = InstrumentServer().init()
    gpu(rpc)
    rpc.stop()



from distutils.version import LooseVersion

from ..servers.dvt import DTXServer
from ..util.exceptions import StartServiceError
from ..util.lockdown import LockdownClient


class TestManagerdLockdown(DTXServer):
    def __init__(self, lockdown=None,udid=None,*args, **kw):
        super().__init__(*args, **kw)
        self.lockdown = lockdown or LockdownClient(udid=udid)

    def init(self,_cli=None):
        """
        初始化 servers rpc 服务:
        :return: bool 是否成功
        """
        try:
            if self.lockdown.ios_version >= LooseVersion('14.0'):
                self._cli = self.lockdown.start_service("com.apple.testmanagerd.lockdown.secure")
            else:
                self._cli = self.lockdown.start_service("com.apple.testmanagerd.lockdown")
                if hasattr(self._cli.sock, '_sslobj'):
                    self._cli.sock._sslobj = None  # remoteserver 协议配对成功之后，需要关闭 ssl 协议通道，使用明文传输
        except StartServiceError as E:
            raise E
        if self._cli is None:
            return False
        self._start()
        return self



#!/usr/bin/env python3
# Referenced from
# https://github.com/doronz88/pymobiledevice3/blob/5003eb82b45a3da9ef2c472c88361ec454497469/pymobiledevice3/services/os_trace.py
import logging
import plistlib
import struct
from abc import ABC
from datetime import datetime

from construct import Struct, Bytes, Int32ul, CString, Optional, Enum, Byte, Adapter, Int16ul, this, Computed

from ios_device.util.lockdown import LockdownClient

CHUNK_SIZE = 4096
TIME_FORMAT = '%H:%M:%S'
SYSLOG_LINE_SPLITTER = '\n\x00'


class TimestampAdapter(Adapter, ABC):
    def _decode(self, obj, context, path):
        return datetime.fromtimestamp(obj.seconds + (obj.microseconds / 1000000))

    def _encode(self, obj, context, path):
        return list(map(int, obj.split(".")))


timestamp_t = Struct(
    'seconds' / Int32ul,
    Bytes(4),
    'microseconds' / Int32ul
)


def try_decode(s):
    try:
        return s.decode('utf8')
    except UnicodeDecodeError:
        return s


syslog_t = Struct(
    Bytes(9),
    'pid' / Int32ul,
    Bytes(42),
    'timestamp' / TimestampAdapter(timestamp_t),
    Bytes(1),
    'level' / Enum(Byte, Notice=0, Info=0x01, Debug=0x02, Error=0x10, Fault=0x11),
    Bytes(38),
    'image_name_size' / Int16ul,
    'message_size' / Int16ul,
    Bytes(19),
    'filename' / CString('utf8'),
    '_image_name' / Bytes(this.image_name_size),
    'image_name' / Computed(lambda ctx: try_decode(ctx._image_name[:-1])),
    '_message' / Bytes(this.message_size),
    'message' / Computed(lambda ctx: try_decode(ctx._message[:-1])),
    'label' / Optional(Struct(
        'bundle_id' / CString('utf8'),
        'identifier' / CString('utf8')
    )),
)


class OsTraceService(object):
    SERVICE_NAME = 'com.apple.os_trace_relay'

    def __init__(self, lockdown: LockdownClient=None):
        self.logger = logging.getLogger(__name__)
        self.lockdown = lockdown or LockdownClient()
        self.c = self.lockdown.start_service(self.SERVICE_NAME)

    def get_pid_list(self):
        self.c.send_plist({'Request': 'PidList'})
        self.c.recv_exact(1)
        return self.c.recv_plist()

    def syslog(self, pid=-1):
        self.c.send_plist({'Request': 'StartActivity', 'MessageFilter': 65535, 'Pid': pid, 'StreamFlags': 60})

        length_length, = struct.unpack('<I', self.c.recv_exact(4))
        length = int(self.c.recv_exact(length_length)[::-1].hex(), 16)
        print("length_length",length)
        response = plistlib.loads(self.c.recv_exact(length))

        if response.get('Status') != 'RequestSuccessful':
            raise Exception(f'got invalid response: {response}')

        while True:
            self.c.recv_exact(1)
            length, = struct.unpack('<I', self.c.recv_exact(4))
            line = self.c.recv(length)
            entry = syslog_t.parse(line)
            yield entry




from distutils.version import LooseVersion

from ..servers.dvt import DTXServer
from ..util import Log
from ..util.exceptions import StartServiceError
from ..util.lockdown import LockdownClient
from ..util.variables import LOG

log = Log.getLogger(LOG.Instrument.value)


class InstrumentServer(DTXServer):
    def __init__(self, lockdown=None, udid=None, network=None, *args, **kw):
        super().__init__(*args, **kw)
        self.lockdown = lockdown or LockdownClient(udid=udid, network=network)

    def init(self, _cli=None):
        """
        初始化 servers rpc 服务:
        :return: bool 是否成功
        """
        log.info('InstrumentServer init ...')
        if not _cli:
            try:
                if self.lockdown.ios_version >= LooseVersion('14.0'):
                    self._cli = self.lockdown.start_service("com.apple.instruments.remoteserver.DVTSecureSocketProxy")
                else:
                    self._cli = self.lockdown.start_service("com.apple.instruments.remoteserver")
                    if hasattr(self._cli.sock, '_sslobj'):
                        self._cli.sock._sslobj = None  # remoteserver 协议配对成功之后，需要关闭 ssl 协议通道，使用明文传输
            except StartServiceError as E:
                raise E
        else:
            self._cli = _cli
        self._start()
        if self._cli is None:
            return False
        return self





import logging

from ios_device.util.lockdown import LockdownClient

notifications = ['com.apple.security.secureobjectsync.viewschanged', 'com.apple.AOSNotification.FMIPStateDidChange',
                 'com.apple.assistant.siri_settings_did_change', 'CNFavoritesChangedExternallyNotification',
                 'com.apple.security.view-change.PCS', 'com.apple.ap.adprivacyd.reconcile',
                 'com.apple.duetexpertd.ATXAnchorModel.WiredAudioDeviceConnectedAnchor',
                 'com.apple.ams.privateListeningChanged',
                 'com.apple.rapport.prefsChanged', 'com.apple.locationd.authorization',
                 'com.apple.cmfsyncagent.storedidchangeexternally',
                 'com.apple.homed.user-cloud-share.wake.com.apple.siri.zonesharing',
                 'com.apple.ProtectedCloudStorage.mobileBackupStateChange',
                 'com.apple.MobileAsset.CoreTextAssets.ma.new-asset-installed',
                 'com.apple.coreduet.idslaunchnotification',
                 'com.apple.accessibility.cache.enhance.text.legibility', 'com.apple.system.config.network_change',
                 'com.apple.MobileAsset.VoiceTriggerAssetsIPad.ma.new-asset-installed',
                 'com.apple.LaunchServices.applicationUnregistered',
                 'com.apple.coreduetd.knowledgebase.launch.duetexpertd',
                 'com.apple.homed.user-cloud-share.repair.wake.com.apple.applemediaservices.multiuser',
                 'com.apple.tv.appRemoved',
                 'com.apple.MobileSoftwareUpdate.OSVersionChanged', 'AppleNumberPreferencesChangedNotification',
                 'com.apple.isp.frontcamerapower', 'HKHealthDaemonActiveWorkoutServersDidUpdateNotification',
                 'com.apple.networkserviceproxy.reset', 'com.apple.appstored.PodcastSubEntitlementsCacheUpdated',
                 'com.apple.homed.televisionAccessoryAdded', 'com.apple.sbd.kvstorechange',
                 'com.apple.MobileAsset.VoiceTriggerAssetsWatch.ma.new-asset-installed',
                 'com.apple.ContinuityKeyBoard.enabled',
                 'CKIdentityUpdateNotification', 'com.apple.assistant.speech-capture.finished',
                 'com.apple.proactive.PersonalizationPortrait.namedEntitiesDidChangeMeaningfully',
                 '__ABDataBaseChangedByOtherProcessNotification', 'com.apple.navd.wakeUpForHypothesisUpdate',
                 'com.apple.language.changed', 'com.apple.family.family_updated',
                 'com.apple.duetexpertd.mm.audiodisconnect',
                 'com.apple.itunesstored.accountschanged', 'com.apple.EscrowSecurityAlert.record',
                 'com.apple.AirTunes.DACP.play',
                 'com.apple.alarm.label.siri_data_changed', 'com.apple.UsageTrackingAgent.registration.video',
                 'com.apple.StoreServices.StorefrontChanged',
                 'com.apple.accessibility.cache.darken.system.colors.enabled',
                 'com.apple.trial.NamespaceUpdate.SIRI_UNDERSTANDING_ASR_ASSISTANT',
                 'com.apple.VideoSubscriberAccount.DidRegisterSubscription', 'com.apple.system.clock_set',
                 'com.apple.media.podcasts.siri_data_changed', 'com.apple.networkextension.apps-changed',
                 'kAFPreferencesDidChangeDarwinNotification', 'com.apple.callhistory.RecentsClearedNotification',
                 'MISProvisioningProfileRemoved', 'com.apple.springboard.finishedstartup',
                 'com.apple.welcomemat.finalizeMigratableApps', 'com.apple.coreaudio.borealisTrigger',
                 'com.apple.MobileAsset.VoiceTriggerAssetsWatch.new-asset-installed',
                 'com.apple.cddcommunicator.pluginChanged',
                 'com.apple.security.view-change.SE-PTC', 'com.apple.voicetrigger.enablePolicyChanged',
                 'com.apple.CloudSubscriptionFeature.Changed', 'com.apple.security.cloudkeychainproxy.kvstorechange3',
                 'com.apple.AirTunes.DACP.devicevolume', 'com.apple.homed.AppleTVAccessoryAdded',
                 'kKeepAppsUpToDateEnabledChangedNotification', 'com.apple.security.octagon.trust-status-change',
                 'com.apple.siri.inference.biome-context', 'com.apple.tv.updateAppVisibility',
                 'com.apple.sockpuppet.applications.updated',
                 'com.apple.softwareupdateservicesd.SUCoreConfigScheduledScan',
                 'com.apple.MobileAsset.CoreTextAssets.ma.cached-metadata-updated',
                 'com.apple.ap.adprivacyd.iTunesActiveStorefrontDidChangeNotification',
                 'com.apple.appstored.AppStoreSubEntitlementsCacheUpdated',
                 'com.apple.remotemanagement.accountsChanged',
                 'com.apple.coreduet.client-needs-help.coreduetd', 'com.apple.LaunchServices.ApplicationsChanged',
                 'com.apple.navd.backgroundCommute.startPredicting', 'com.apple.suggestions.prepareForQuery',
                 'com.apple.LaunchServices.applicationRegistered', 'com.apple.icloud.fmip.siri_data_changed',
                 'com.apple.carkit.carplay-attached', 'com.apple.photostream.idslaunchnotification',
                 'com.apple.voicemail.ReloadService', 'com.apple.homed.speakersConfiguredChanged',
                 'com.apple.duetexpertd.ATXMMAppPredictor.WiredAudioDeviceDisconnectedAnchor',
                 'com.apple.icloud.fmip.lostmode.enable',
                 'com.apple.pasteboard.notify.changed', 'com.apple.icloudpairing.idslaunchnotification',
                 'com.apple.remindd.nano_preferences_sync', 'kVMVoicemailTranscriptionTaskTranscribeAllVoicemails',
                 'com.apple.bluetooth.daemonStarted', 'com.apple.security.view-ready.SE-PTC',
                 'com.apple.system.logging.power_button_notification',
                 'com.apple.MobileAsset.VoiceTriggerAssetsMarsh.ma.new-asset-installed',
                 'com.apple.MobileBackup.backgroundCellularAccessChanged', 'com.apple.nanoregistry.devicedidunpair',
                 'com.apple.awd.launch.wifi', 'com.apple.OTACrashCopier.SubmissionPreferenceChanged',
                 'com.apple.aggregated.addaily.logging', 'com.apple.atc.xpc.runkeeplocaltask',
                 'com.apple.springboard.hasBlankedScreen',
                 'com.apple.MobileAsset.TimeZoneUpdate.ma.cached-metadata-updated',
                 'com.apple.security.publickeynotavailable',
                 'com.apple.shortcuts.daemon-wakeup-request', 'com.apple.cmfsyncagent.kvstorechange',
                 'com.apple.dmd.iCloudAccount.didChange',
                 'com.apple.trial.NamespaceUpdate.SIRI_VALUE_INFERENCE_CONTACT_RESOLUTION',
                 'kCalBirthdayDefaultAlarmChangedNote', 'com.apple.voicetrigger.PHSProfileModified',
                 'com.apple.UsageTrackingAgent.registration.application', 'FMFMeDeviceChangedNotification',
                 'com.apple.mobilecal.preference.notification.weekStart',
                 'com.apple.duetexpertd.ATXMMAppPredictor.BluetoothDisconnectedAnchor',
                 'com.apple.softwareupdateservicesd.activity.autoScan', 'com.apple.siri.preheat.quiet',
                 'com.apple.voicemail.VVVerifierCheckpointDictionaryChanged',
                 'com.apple.SafariShared.Assistant.reload_plugin',
                 'com.apple.reminder.list.name.siri_data_changed', 'com.apple.purplebuddy.setupexited',
                 'com.apple.calendar.database.preference.notification.kCalPreferredDaysToSyncKey',
                 'com.apple.softwareupdateservicesd.activity.autoDownloadEnd',
                 'com.apple.system.batterysavermode.auto_disabled',
                 'com.apple.security.secureobjectsync.circlechanged', 'NewOperatorNotification',
                 'com.apple.spotlight.SyndicatedContentDeleted',
                 'com.apple.MobileAsset.ProactiveEventTrackerAssets.ma.cached-metadata-updated',
                 'com.apple.appstored.ActivitySubEntitlementsCacheUpdated', 'com.apple.suggestions.settingsChanged',
                 'com.apple.voicetrigger.RemoteDarwin.ConnectionChanged',
                 'com.apple.icloud.findmydeviced.findkit.magSafe.added',
                 'com.apple.locationd.appreset',
                 'com.apple.MobileAsset.VoiceServicesVocalizerVoice.ma.cached-metadata-updated',
                 'com.apple.assistant.sync_needed', 'com.apple.pushproxy.idslaunchnotification',
                 'com.apple.nfcacd.multitag.state.change',
                 'com.apple.MobileAsset.SecureElementServiceAssets.ma.cached-metadata-updated',
                 'com.apple.duetexpertd.mm.bluetoothdisconnect', 'com.apple.system.batterysavermode.first_time',
                 'com.apple.touchsetupd.launch', 'com.apple.homehubd.endpointDeactivated',
                 'NoteContextDarwinNotificationWithLoggedChanges', 'com.apple.mobileipod.librarychanged',
                 'com.apple.UsageTrackingAgent.registration.web-domain', 'com.apple.siri.cloud.synch.changed',
                 'com.apple.proactive.queries.databaseChange', 'com.apple.smartcharging.defaultschanged',
                 'com.apple.screensharing.idslaunchnotification', 'FMFDevicesChangedNotification',
                 'com.apple.GeoServices.countryCodeChanged', 'com.apple.powerui.smartcharge',
                 'com.apple.MobileAsset.VoiceServices.GryphonVoice.ma.new-asset-installed',
                 'com.apple.assistant.sync_homekit_now',
                 'com.apple.icloud.findmydeviced.findkit.magSafe.attach',
                 'com.apple.icloud.searchparty.accessoryDidPair',
                 'com.apple.SensorKit.als', 'com.apple.voicetrigger.RemoteDarwin.EarlyDetect',
                 'com.apple.telephonyutilities.callservicesd.fakeincomingmessage',
                 'com.apple.nanoregistry.paireddevicedidchangecapabilities',
                 'com.apple.voiceservices.notification.voice-update',
                 'com.apple.duetbm.internalSettingsChanged',
                 'com.apple.nanophotos.prefs.LibraryCollectionTargetMapData-changed',
                 'com.apple.pex.connections.focalappchanged',
                 'com.apple.accessibility.cache.enhance.background.contrast',
                 'com.apple.MobileAsset.VoiceServices.VoiceResources.ma.new-asset-installed',
                 'com.apple.idscredentials.idslaunchnotification',
                 'com.apple.accessories.connection.MFi4AccessoryDisconnected',
                 'com.apple.mobileme.fmf1.allowFindMyFriendsModification',
                 'com.apple.MobileAsset.TimeZoneUpdate.ma.new-asset-installed',
                 'com.apple.homed.user-cloud-share.wake.com.apple.mediaservicesbroker.container',
                 'com.apple.contacts.clientDidDisplayFavorites', 'com.apple.bluetooth.WirelessSplitterOn',
                 'com.apple.mobileipod.displayvalueschanged', 'com.apple.coremedia.carplayisconnected',
                 'com.apple.duetexpertd.donationmonitor.intent', 'com.apple.mobiletimerd.wakeuptest',
                 'com.apple.powermanagement.systempowerstate', 'com.apple.AppleMediaServices.deviceOffersChanged',
                 'com.apple.AirTunes.DACP.shuffletoggle', 'com.apple.AirTunes.DACP.previtem',
                 'com.apple.powermanagement.restartpreventers', 'com.apple.SensorKit.phoneUsageReport',
                 'com.apple.awd.launch.nfcd',
                 'com.apple.private.SensorKit.pedometer.stridecalibration', 'com.apple.mobile.disk_image_mounted',
                 'com.apple.ManagedConfiguration.webContentFilterTypeChanged',
                 'com.apple.ProximityControl.LockScreenDiscovery',
                 'com.apple.awdd.anonymity', 'com.apple.sleep.sync.SleepScheduleDidChange',
                 'com.apple.mobileipod.keeplocalstatechanged', 'com.apple.assistant.sync_data_changed',
                 'com.apple.locationd.vehicle.exit', 'com.apple.carkit.capabilities-changed',
                 'com.apple.parsecd.queries.clearData',
                 'com.apple.parsecd.bag', 'com.apple.MobileAsset.VoiceTriggerAssetsMarsh.ma.cached-metadata-updated',
                 'com.apple.duetexpertd.dockAppListCacheUpdate', 'com.apple.itunesstored.autodownloaddefaultschange',
                 'com.apple.powerlog.batteryServiceNotification', 'AppleLanguagePreferencesChangedNotification',
                 'com.apple.voiceservices.trigger.asset-force-update', 'com.apple.itunesstored.invalidatebags',
                 'com.apple.duetexpertd.defaultsChanged', 'com.apple.security.itembackup',
                 'com.apple.powermanagement.idlesleeppreventers', 'com.apple.system.batterysavermode',
                 'SBApplicationNotificationStateChanged', 'com.apple.proactive.queries.clearData',
                 'com.apple.coreduetd.nearbydeviceschanged', 'com.apple.trial.NamespaceUpdate.FREEZER_POLICIES',
                 'com.apple.duetexpertd.ATXMMAppPredictor.CarPlayDisconnectedAnchor',
                 'com.apple.EscrowSecurityAlert.server',
                 'com.apple.siri.cloud.storage.deleted', 'com.apple.thermalmonitor.ageAwareMitigationsEnabled',
                 'com.apple.corespotlight.developer.ReindexAllItemsWithIdentifiers',
                 'com.apple.system.powersources.percent',
                 'com.apple.MusicLibrary.importFinished-/var/mobile/Media/iTunes_Control/iTunes/MediaLibrary.sqlitedb',
                 'com.apple.ProtectedCloudStorage.rollNow', 'com.apple.mobile.lockdown.device_name_changed',
                 'com.apple.mobilecal.timezonechanged', 'com.apple.mobileslideshow.ICPLStateChanged',
                 'com.apple.MobileAsset.VoiceTriggerAssets.cached-metadata-updated', 'com.apple.springboard.lockstate',
                 'com.apple.MobileAsset.SpeechEndpointAssets.ma.cached-metadata-updated',
                 'SignificantTimeChangeNotification',
                 'com.apple.trial.NamespaceUpdate.SIRI_DICTATION_ASSETS', 'com.apple.security.tick',
                 'VVMessageWaitingFallbackNotification', 'com.apple.ProtectedCloudStorage.rollIfAged',
                 'MISProvisioningProfileInstalled', 'com.apple.coreduetd.remoteDeviceChange',
                 'kFZACAppBundleIdentifierLaunchNotification',
                 'com.apple.datamigrator.datamigrationcompletecontinuerestore',
                 'com.apple.fairplayd.resync-fpkeybag', 'com.apple.mobiletimerd.resttest',
                 'com.apple.MobileAsset.VoiceTriggerAssets.ma.cached-metadata-updated',
                 'com.apple.idsremoteurlconnection.idslaunchnotification', 'com.apple.homehubd.endpointActivated',
                 'com.apple.locationd.vehicle.connected',
                 'com.apple.MobileAsset.SpeechEndpointAssets.cached-metadata-updated',
                 'com.apple.dataaccess.ping', 'com.apple.GeoServices.navigation.stopped',
                 'com.apple.softwareupdateservicesd.activity.emergencyAutoScan',
                 'com.apple.idstransfers.idslaunchnotification',
                 'com.apple.softwareupdateservicesd.activity.autoInstallUnlock',
                 'AFLanguageCodeDidChangeDarwinNotification',
                 'RTLocationsOfInterestDidChangeNotification', 'com.apple.mobiletimerd.chargetest',
                 'com.apple.VideosUI.StoreAcquisitionCrossProcessNotification',
                 'com.apple.system.powermanagement.useractivity2',
                 'com.apple.duetexpertd.ATXAnchorModel.invalidate.BluetoothConnectedAnchor',
                 'com.apple.duetexpertd.clientModelRefreshBlendingLayer', 'com.apple.chatkit.groups.siri_data_changed',
                 'com.apple.corespotlight.developer.ReindexAllItems', 'kFZVCAppBundleIdentifierLaunchNotification',
                 'AppleDatePreferencesChangedNotification', 'INVoocabularyChangedNotification',
                 'com.apple.telephonyutilities.callservicesdaemon.voicemailcallended',
                 'com.apple.MediaRemote.nowPlayingActivePlayersIsPlayingDidChange',
                 'com.apple.nanoregistry.watchdidbecomeactive',
                 'com.apple.exchangesyncd.ping', 'com.apple.networkextension.app-paths-changed',
                 'com.apple.nearfield.handoff.terminal',
                 'com.apple.cloudd.pcsIdentityUpdate-com.apple.ProactivePredictionsBackup',
                 'com.apple.cloudrecents.kvstorechange',
                 'com.apple.mobileipod.libraryimportdidfinish', 'com.apple.mobiletimerd.reset',
                 'com.apple.duetexpertd.homeScreenPageConfigCacheUpdate', 'com.apple.da.tasking_changed',
                 'com.apple.LoginKit.isLoggedIn', 'com.apple.duetexpertd.ATXAnchorModel.invalidate.IdleTimeEndAnchor',
                 'com.apple.security.cloudkeychain.forceupdate', '_CalDatabaseChangedNotification',
                 'AppleKeyboardsPreferencesChangedNotification',
                 'com.apple.UsageTrackingAgent.registration.now-playing', 'com.apple.cddcommunicator.batteryChanged',
                 'SLSharedWithYouSettingHasChanged', 'com.apple.pairedsync.syncDidComplete',
                 'com.apple.powerui.requiredFullCharge',
                 'CKAccountChangedNotification',
                 'com.apple.duetexpertd.ATXMMAppPredictor.WiredAudioDeviceConnectedAnchor',
                 'com.apple.duetexpertd.ATXAnchorModel.BluetoothConnectedAnchor',
                 'com.apple.hangtracerd.htse_state_changed',
                 'com.apple.AirTunes.DACP.repeatadv', 'com.apple.GeoServices.navigation.started',
                 'com.apple.spotlightui.prefschanged',
                 'com.apple.MobileAsset.VoiceServices.VoiceResources.new-asset-installed', 'logging tasks have changed',
                 'EKNotificationCountChangedExternallyNotification', 'com.apple.duetexpertd.donationmonitor.activity',
                 'com.apple.ProtectedCloudStorage.updatedKeys',
                 'com.apple.softwareupdateservicesd.activity.presentBanner',
                 'com.apple.ProtectedCloudStorage.rollBackupDisabled',
                 'com.apple.appstored.iCloudSubEntitlementsCacheUpdated',
                 'com.apple.stockholm.se.mfd', 'com.apple.mobiletimerd.diagnostics',
                 'com.apple.mobile.keybagd.first_unlock',
                 'com.apple.proactive.information.source.weather', 'com.apple.icloud.searchparty.selfbeaconchanged',
                 'com.apple.accessibility.cache.reduce.motion', 'com.apple.locationd.vehicular.changed.toVehicular',
                 'com.apple.bluetooth.accessory-authentication.success', 'com.apple.locationd/Prefs',
                 'com.apple.spotlight.SyndicatedContentRefreshed', 'com.apple.networkextension.nehelper-init',
                 'com.apple.managedconfiguration.allowpasscodemodificationchanged',
                 'com.apple.DuetHeuristic-BM.shutdowsoon',
                 'com.apple.system.powersources.source', 'com.apple.managedconfiguration.effectivesettingschanged',
                 'com.apple.datamigrator.migrationDidFinish', 'com.apple.nanomusic.sync.defaults',
                 'com.apple.homed.multi-user-status-changed',
                 'com.apple.siri.ShortcutsCloudKitAccountModifiedNotification',
                 'com.apple.mobilemail.afc.poll', 'com.apple.appstored.TVSubEntitlementsCacheUpdated',
                 'com.apple.duetexpertd.ATXAnchorModel.invalidate.ChargerConnectedAnchor',
                 'com.apple.ManagedConfiguration.profileListChanged', 'com.apple.duetexpertd.ms.nowplayingpause',
                 'com.apple.devicemanagementclient.longLivedTokenChanged',
                 'com.apple.MCX._managementStatusChangedForDomains',
                 'com.apple.triald.new-experiment', 'com.apple.security.secureobjectsync.holdlock',
                 'CNContactStoreDidChangeNotification', 'kFaceTimeChangedNotification',
                 'com.apple.nanoregistry.devicedidpair',
                 'com.apple.wirelessproximity.launch', 'com.apple.wcd.wake-up',
                 'com.apple.icloud.findmydeviced.localActivationLockInfoChanged', 'FMFDataUpdateCompleteNotification',
                 'com.apple.softwareupdateservicesd.activity.autoInstallEnd', 'com.apple.security.publickeyavailable',
                 'MFNanoMailImportantBridgeSettingHasChangedDarwinNotification',
                 'com.apple.duetexpertd.ms.carplaydisconnect',
                 'com.apple.MobileAsset.SecureElementServiceAssets.ma.new-asset-installed',
                 'com.apple.symptoms.materialLinkQualityChange', 'com.apple.ams.provision-biometrics',
                 'com.apple.ap.adprivacyd.canceltasks', 'com.apple.AirTunes.DACP.pause', 'com.apple.audio.AOP.enable',
                 'CalSyncClientBeginningMultiSave', 'com.apple.system.powersources.timeremaining',
                 'com.apple.mobiletimerd.bedtimetest',
                 'com.apple.tv.TVWidgetExtension.Register', 'com.apple.SensorKit.deviceUsageReport',
                 'com.apple.mobileipod-prefsChanged', 'com.apple.purplebuddy.setupdone', 'NewCarrierNotification',
                 'com.apple.bluetooth.pairing', 'com.apple.healthlite.SleepDetectedActivity',
                 'com.apple.mobilecal.invitationalertschanged', 'CalSyncClientFinishedMultiSave',
                 'com.apple.parsec-fbf.FLUploadImmediately',
                 'com.apple.MobileAsset.AppleKeyServicesCRL.new-asset-installed',
                 'com.apple.locationd.vehicle.disconnected', 'com.apple.sleepd.diagnostics',
                 'com.apple.media.entities.siri_data_changed', 'com.apple.appstored.MusicSubEntitlementsCacheUpdated',
                 'com.apple.MobileAsset.ProactiveEventTrackerAssets.ma.new-asset-installed',
                 'com.apple.icloud.findmydeviced.findkit.magSafe.detach', 'com.apple.AirTunes.DACP.volumeup',
                 'com.apple.siri.ShortcutsCloudKitAccountAddedNotification', 'com.apple.tcc.access.changed',
                 'com.apple.Music-AllowsCellularDataDownloads', 'com.apple.videos.migrationCompleted',
                 'com.apple.assistant.app_vocabulary', 'com.apple.SensorKit.visits',
                 'com.apple.managedconfiguration.passcodechanged',
                 'com.apple.icloud.FindMy.addMagSafeAccessory', 'com.apple.duetexpertd.appchangeprediction',
                 'com.apple.telephonyutilities.callservicesd.fakeoutgoingmessage',
                 'com.apple.MediaRemote.nowPlayingApplicationIsPlayingDidChange',
                 'com.apple.duetexpertd.ATXAnchorModel.ChargerConnectedAnchor',
                 'kCalEventOccurrenceCacheChangedNotification',
                 'com.apple.TVRemoteCore.connectionRequested', 'com.apple.homekit.sync-data-cache-updated',
                 'com.apple.duetexpertd.mm.bluetoothconnected', 'com.apple.mobiletimerd.waketest',
                 'com.apple.ap.adprivacyd.launch',
                 'com.apple.system.lowpowermode.auto_disabled', 'com.apple.ap.adprivacyd.deviceKnowledge',
                 'com.apple.duetexpertd.ATXMMAppPredictor.BluetoothConnectedAnchor',
                 'com.apple.system.powermanagement.poweradapter',
                 'com.apple.MobileAsset.VoiceTriggerAssets.new-asset-installed',
                 'com.apple.accessibility.cache.invert.colors',
                 'com.apple.healthlite.SleepSessionEndRequest', 'com.apple.geoservices.siri_data_changed',
                 'com.apple.appletv.backgroundstate', 'MPStoreClientTokenDidChangeNotification',
                 'com.apple.SensorKit.messagesUsageReport', 'com.apple.imautomatichistorydeletionagent.prefchange',
                 'com.apple.homed.user-cloud-share.wake.com.apple.siri.data', 'com.apple.dmd.budget.didChange',
                 'com.apple.NanoPhotos.Library.changed',
                 'com.apple.duetexpertd.ATXAnchorModel.invalidate.WiredAudioDeviceConnectedAnchor',
                 'com.apple.assistant.sync_homekit_urgent',
                 'com.apple.calendar.database.preference.notification.suggestEventLocations',
                 'com.apple.accessibility.cache.differentiate.without.color', 'EKFeatureSetDidChangeNotification',
                 'PCPreferencesDidChangeNotification',
                 'com.apple.duetexpertd.ATXAnchorModel.invalidate.CarPlayConnectedAnchor',
                 'com.apple.MobileAsset.VoiceTriggerAssetsWatch.ma.cached-metadata-updated',
                 'com.apple.rapport.CompanionLinkDeviceAdded', 'com.apple.cddcommunicator.nwchanged',
                 'com.apple.duetexpertd.ATXMMAppPredictor.CarPlayConnectedAnchor',
                 'com.apple.system.powersources.criticallevel',
                 'com.apple.BiometricKit.passcodeGracePeriodChanged', 'com.apple.powermanagement.systemsleeppreventers',
                 'SLSharedWithYouAppSettingHasChanged', 'com.apple.ManagedConfiguration.webContentFilterChanged',
                 'com.apple.duetexpertd.updateDefaultsDueToRelevantHomeScreenConfigUpdate',
                 'FMFFollowersChangedNotification',
                 'com.apple.softwareupdateservicesd.activity.installAlert',
                 'ConnectedGymPreferencesChangedNotification',
                 'com.apple.voicetrigger.XPCRestarted', 'com.apple.softwareupdateservicesd.activity.delayEndScan',
                 'com.apple.duetexpertd.ATXScreenUnlockUpdateSource',
                 'com.apple.appstored.NewsSubEntitlementsCacheUpdated',
                 'com.apple.fitness.FitnessAppInstalled', 'com.apple.managedconfiguration.restrictionchanged',
                 'com.apple.nanoregistry.paireddevicedidchangeversion', 'com.apple.iokit.hid.displayStatus',
                 'com.apple.coreaudio.RoutingConfiguration', 'com.apple.mobileipod.noncontentspropertieschanged',
                 'com.apple.GeoServices.PreferencesSync.SettingsChanged', 'com.apple.isp.backcamerapower',
                 'com.apple.duetexpertd.ATXAnchorModel.CarPlayConnectedAnchor',
                 'BYSetupAssistantFinishedDarwinNotification',
                 'com.apple.springboard.pluggedin', 'com.apple.AirTunes.DACP.device-prevent-playback',
                 'com.apple.duetexpertd.ms.carplayconnect', 'com.apple.accessories.connection.passedMFi4Auth',
                 'CNContactStoreMeContactDidChangeNotification', 'com.apple.MediaRemote.nowPlayingInfoDidChange',
                 'com.apple.system.thermalpressurelevel', 'com.apple.mobile.lockdown.activation_state',
                 'com.apple.homed.user-cloud-share.wake.com.apple.applemediaservices.multiuser.qa',
                 'com.apple.AirTunes.DACP.devicevolumechanged', 'com.apple.system.hostname',
                 'com.apple.system.lowpowermode.first_time',
                 'com.apple.mobileslideshow.PLNotificationKeepOriginalsChanged',
                 'com.apple.mobilecal.preference.notification.calendarsExcludedFromNotifications',
                 'com.apple.MobileAsset.VoiceTriggerAssetsIPad.ma.cached-metadata-updated',
                 'com.apple.duetexpertd.feedbackavailable',
                 'com.apple.MobileAsset.VoiceTriggerAssets.ma.new-asset-installed',
                 'com.apple.duetexpertd.ms.nowplayingplay',
                 'com.apple.CallHistoryPluginHelper.launchnotification', 'com.apple.cddcommunicator.thermalChanged',
                 'com.apple.ManagedConfiguration.managedAppsChanged',
                 'com.apple.managedconfiguration.managedorginfochanged',
                 'com.apple.system.powermanagement.uservisiblepowerevent', 'com.apple.AirTunes.DACP.nextitem',
                 'com.apple.bookmarks.BookmarksFileChanged', 'com.apple.duetexpertd.prefschanged',
                 'com.apple.mobiletimerd.goodmorningtest', 'com.apple.bluetooth.connection',
                 'com.apple.VideosUI.UpNextRequestDidFinishNotification',
                 'com.apple.VideosUI.PlayHistoryUpdatedNotification',
                 'com.apple.proactive.PersonalizationPortrait.namedEntitiesInvalidated',
                 'com.apple.itunescloudd.artworkDownloadsDidCompleteNotification',
                 'com.apple.duetexpertd.ATXMMAppPredictor.IdleTimeEndAnchor',
                 'com.apple.Preferences.ResetPrivacyWarningsNotification',
                 'com.apple.siri.client.state.DynamiteClientState.siri_data_changed',
                 'com.apple.Preferences.ChangedRestrictionsEnabledStateNotification', 'com.apple.system.timezone',
                 'com.apple.commcenter.InternationalRoamingEDGE.changed', 'com.apple.duetexpertd.appclipprediction',
                 'com.apple.homed.user-cloud-share.wake.com.apple.applemediaservices.multiuser',
                 'com.apple.icloud.findmydeviced.findkit.magSafe.removed',
                 'com.apple.MobileAsset.Font7.ma.cached-metadata-updated',
                 'ApplePreferredContentSizeCategoryChangedNotification',
                 'com.apple.softwareupdate.autoinstall.startInstall',
                 'com.apple.corerecents.iCloudAccountChanged',
                 'com.apple.trial.NamespaceUpdate.NETWORK_SERVICE_PROXY_CONFIG_UPDATE',
                 'ACDAccountStoreDidChangeNotification', 'com.apple.triald.wake', 'VMStoreSetTokenNotification',
                 'com.apple.ap.adprivacyd.iTunesActiveAccountDidChangeNotification', 'com.apple.system.lowpowermode',
                 'com.apple.duet.expertcenter.appRefresh', 'com.apple.Sharing.prefsChanged',
                 'com.apple.system.thermalpressurelevel.cold', 'com.apple.siri.inference.coreduet-context',
                 'com.apple.mobile.keybagd.lock_status', 'com.apple.MobileAsset.EmbeddedSpeech.ma.new-asset-installed',
                 'com.apple.dataaccess.checkHolidayCalendarAccount',
                 'com.apple.softwareupdateservicesd.activity.autoDownload',
                 'com.apple.callhistorysync.idslaunchnotification',
                 'com.apple.ProtectedCloudStorage.test.mobileBackupStateChange',
                 'com.apple.voicetrigger.EarlyDetect', 'com.apple.AirTunes.DACP.volumedown',
                 'com.apple.timezonesync.idslaunchnotification', 'AppleTimePreferencesChangedNotification',
                 'com.apple.AirTunes.DACP.mutetoggle', 'com.apple.EscrowSecurityAlert.reset',
                 'com.apple.Carousel.wristStateChanged',
                 'com.apple.voicemail.changed', 'com.apple.duetexpertd.ATXAnchorModel.IdleTimeEndAnchor',
                 'dmf.policy.monitor.app',
                 'com.apple.coreaudio.speechDetectionVAD.created']


class NotificationProxyService(object):
    SERVICE_NAME = 'com.apple.mobile.notification_proxy'

    def __init__(self, lockdown=None, udid=None, network=None, logger=None):
        self.logger = logger or logging.getLogger(__name__)
        self.lockdown = lockdown or LockdownClient(udid=udid, network=network)
        self.service = self.lockdown.start_service(self.SERVICE_NAME)

    def notify_post(self, name: str):
        self.service.send_plist({'Command': 'PostNotification',
                                 'Name': name})

    def notify_observe(self, name: str):
        self.logger.info('Observing %s', name)
        self.service.send_plist({'Command': 'ObserveNotification',
                                 'Name': name})

    def receive_notification(self):
        while True:
            yield self.service.recv_plist()


if __name__ == '__main__':
    service = NotificationProxyService()
    for notification in notifications:
        service.notify_observe(notification)
    for i in service.receive_notification():
        print(i)



import io
import struct
import typing
from copy import deepcopy
from dataclasses import dataclass, field

import execjs


@dataclass
class GRCDecodeOrder:
    """ 解码序列
    """
    key: str
    require: bool
    val: int = field(default=None, repr=True)

    @staticmethod
    def decode(data):
        _list = list()
        for _index in data:
            _list.append(GRCDecodeOrder(*_index))
        return _list


@dataclass
class GRCDisplayOrder:
    """ 最终展示显示编码序列
    """
    display: str
    scale: int
    content: str
    method: str
    mix: int
    min: int

    @staticmethod
    def decode(data):
        _list = list()
        for i in data:
            _list.append(GRCDisplayOrder(*i))
        return _list


@dataclass
class TraceData:
    """ 返回 bytes 数据流
    """
    type: int
    time: int
    time_stamp: int
    trace_num: int  # 组数量，每组根据 GRCDecodeOrder 序列进行相关逻辑解析
    trace_data: bytes
    padding:int = field(default=None, repr=False) # 未知填充



class GPUTraceData:
    def __init__(self, time):
        self.time = time
        self.data_list = list()

    def append(self, data: GRCDecodeOrder):
        self.data_list.append(data)

    def get_size(self):
        return len(self.data_list)

    def get(self, index):
        return self.data_list[index]


class GUPCounterData:

    def __init__(self, fast_counter_time, last_counter_time):
        self.fast_counter_time = fast_counter_time
        self.last_counter_time = last_counter_time

        self.counter_data_list = list()

    def append(self, data):
        self.counter_data_list.append(data)

    def get(self, index):
        return self.counter_data_list[index]

    def get_size(self):
        return len(self.counter_data_list)



class JSEvn:

    def __init__(self, js_str, display_key_list, decode_key_list, mach_time_factor):
        self.display_key_list = display_key_list # 显示顺序
        self.decode_key_list = decode_key_list # 解码顺序
        self.mach_time_factor = mach_time_factor
        self.counter_list = None
        self.long_list = None
        self.fast_counter_time = 0

        js = self.format_js(js_str, display_key_list, decode_key_list)
        self.ctx = execjs.compile(js)




    def trace_decode(self,trace_data:TraceData) -> typing.List[GPUTraceData]:
        """ trace 转换成 Long
        :param trace_data:
        :return:
        """
        index = 0
        data_io = io.BytesIO(trace_data.trace_data)  # 二进制流
        data_list = []
        while True:
            index += 1
            d = data_io.read(8)
            if not d:
                break
            _temp_val_list = []
            tmp_data = GPUTraceData(struct.unpack('Q', d)[0])
            for i in self.decode_key_list:
                d = data_io.read(8)
                i.val = struct.unpack('Q', d)[0]
                tmp_data.append(deepcopy(i))
            data_list.append(tmp_data)
        return data_list

    def get_counter_list(self, trace_data:TraceData) -> typing.List[GUPCounterData]:
        counter_list = list()
        index = 0
        fast_counter = None
        trace_list = self.trace_decode(trace_data)
        while index < len(trace_list):
            _counter = deepcopy(trace_list[index])
            tmp_counter =  deepcopy(trace_list[index])
            if index == 0:
                counter_list.append(self._calculation(_counter, _counter))
            else:
                counter_list.append(self._calculation(fast_counter, _counter))
            fast_counter = tmp_counter
            index += 1
        return counter_list

    @staticmethod
    def _calculation(fast_counter: GPUTraceData, last_counter: GPUTraceData) -> GUPCounterData :
        """ 二次处理数据，某些情况需要处理数据差值
        :param fast_counter:
        :param last_counter:
        :return:
        """
        counter_data = GUPCounterData(fast_counter.time, last_counter.time)
        for index, counter in enumerate(last_counter.data_list):
            if not counter.key.startswith('_'):
                counter_data.append(counter)
            else:
                if counter.require:
                    counter_data.append(counter)
                else:
                    counter.val = counter.val - fast_counter.get(index).val
                    counter_data.append(counter)
        return counter_data

    @staticmethod
    def counter_to_js(counter_list:typing.List[GUPCounterData]):
        js_counter_list=[]
        for i in counter_list:
            js_counter_list.append(i.last_counter_time)
            for k in i.counter_data_list:
                js_counter_list.append(k.val)
        return js_counter_list

    def ex_js(self,trace_data:TraceData):
        counter_list = self.get_counter_list(trace_data)
        js_val_list = self.counter_to_js(counter_list)
        counter_result = self.ctx.call('EvaluateGPUCounter', len(counter_list), js_val_list)
        return counter_result,counter_list

    def dump_trace(self,trace_data):


        counter_result,counter_list = self.ex_js(trace_data)

        for i, key in enumerate(counter_list):
            if not self.fast_counter_time :
                self.fast_counter_time  = key.fast_counter_time
            timestamp = (key.last_counter_time - self.fast_counter_time ) * self.mach_time_factor
            timestamp = round(timestamp/1000000000,6)
            start = i * len(self.display_key_list)
            for index, k in enumerate(self.display_key_list):
                formatted_data =f'{timestamp:<10}'+f'{k.display:<45}'+f'{round(counter_result[start + index] * k.mix, 2):<6}'
                print(formatted_data)
            print("-----------------------------------------------------------")

    def format_js(self, js_str, display_key_list, decode_key_list):
        stringBuilder = ''

        stringBuilder += js_str

        for i in decode_key_list:
            stringBuilder += f'var {i.key} = 0;\n'
        stringBuilder += f"var MACH_TIME_FACTOR = {self.mach_time_factor};\n"
        stringBuilder += f"var lastTimestamp = 0;\n"
        stringBuilder += f"var MTLStat_nSec = 0;\n"

        stringBuilder += 'function EvaluateGPUCounter(counterNum,counterResult) {\n'
        stringBuilder += 'var _CounterResult = [];\n'
        stringBuilder += 'for (var index = 0; index < counterNum; ++index) {\n'
        stringBuilder += f'var startIndex = index * {len(decode_key_list)+1} + 1;\n'
        stringBuilder += f'var timestamp = (counterResult[0 + index * {len(decode_key_list)+1}]) * MACH_TIME_FACTOR;\n'
        stringBuilder += f'MTLStat_nSec = timestamp - lastTimestamp;\n'
        stringBuilder += f'var grcGPUCycles = counterResult[1 + startIndex];\n'
        for index, k in enumerate(decode_key_list):
            stringBuilder += f'{k.key} = counterResult[{index} + startIndex];\n'
            stringBuilder += f'{k.key}_norm = counterResult[{index} + startIndex] / grcGPUCycles\n'

        for i in display_key_list:
            stringBuilder += 'try {' + f'value = {i.method}(); ' + '_CounterResult.push(value);} catch(err) {console.error(err); _CounterResult.push(0);}\n'
        stringBuilder += 'lastTimestamp = timestamp\n'
        stringBuilder += '}\n'
        stringBuilder += 'return _CounterResult\n'
        stringBuilder += '}\n'
        return stringBuilder



#!/usr/bin/env python3

import logging
import struct

from construct import Struct, Int32ub, Int32ul, Bytes, Byte, this, Padding, Padded, CString

from ios_device.util.lockdown import LockdownClient

packet_struct = Struct(
    'hdr_len' / Int32ub,
    'hdr_version' / Byte,
    'payload_len' / Int32ub,
    'if_type' / Byte,
    'if_unit' / Padding(2),
    'in_out' / Padding(1),
    'proto_family' / Int32ub,
    'pre_len' / Int32ub,
    'post_len' / Int32ub,
    'interface_name' / Padded(16, CString('utf8')),
    'pid' / Int32ul,
    'comm' / Padded(17, CString('utf8')),
    'svc_class' / Int32ub,
    'epid' / Int32ul,
    'ecomm' / Padded(17, CString('utf8')),
    'epoch_seconds' / Int32ub,
    'epoch_microseconds' / Int32ub,
)


class PcapdService(object):
    IN_OUT_MAP = {0x01: 'O', 0x10: 'I'}
    SERVICE_NAME = "com.apple.pcapd"

    def __init__(self, lockdown=None, udid=None, network=None, logger=None):
        self.logger = logger or logging.getLogger(__name__)
        self.lockdown = lockdown or LockdownClient(udid=udid, network=network)
        self.conn = self.lockdown.start_service(self.SERVICE_NAME)

    def __iter__(self):

        ctp = self._chunk_to_packet

        while True:
            chunk = self.conn.recv_plist()
            yield ctp(chunk)

    def _chunk_to_packet(self, chunk):
        packet = packet_struct.parse(chunk)
        packet.payload = chunk[packet.hdr_len:]

        if not (packet.hdr_version == 2 and len(packet.payload) == packet.payload_len):
            raise ValueError('unsupported version')
        packet.epoch_microseconds = packet.epoch_seconds * 1000000 + packet.epoch_microseconds
        if packet.if_type == 0xFF:
            packet.is_eth = False  # cellular
            packet.payload = packet.payload[4:]
        elif packet.if_type == 0x06:
            packet.is_eth = True  # wifi
        else:
            raise ValueError('unknown link type {}'.format(hex(packet.if_type)))

        return packet


class PCAPPacketDumper:
    HEADER_STRUCT = struct.Struct('=IHHiIII')
    PACKET_STRUCT = struct.Struct('=IIII')

    def __init__(self, pkt_iter, out_file):
        self.pkt_iter = pkt_iter
        self.out_file = out_file

    def run(self, packet_cb=None):
        self.out_file.write(self.HEADER_STRUCT.pack(0xa1b2c3d4, 2, 4, 0, 0, 0xFFFFFFFF, 1))
        for pkt in self.pkt_iter:
            packet_cb is not None and packet_cb(pkt)
            payload = pkt.payload
            if not pkt.is_eth:
                if pkt.proto_family == 2:
                    ether_type_b = b'\x08\x00'  # IPv4
                elif pkt.proto_family == 30:
                    ether_type_b = b'\x86\xDD'  # IPv6
                else:
                    raise NotImplementedError('unsupported proto family {}'.format(pkt.proto_family))
                payload = b''.join((bytes(12), ether_type_b, payload))
            header = self.PACKET_STRUCT.pack(pkt.epoch_microseconds // 1000000,
                                             pkt.epoch_microseconds % 1000000,
                                             len(payload), len(payload))
            self.out_file.writelines((header, payload))




"""
监听事件，比如 app 唤醒，杀死，退出到后台等等
"""

from time import sleep

from ios_device.servers.dvt import DTXServer
from ios_device.servers.Instrument import InstrumentServer


def MobileNotifications(rpc: DTXServer):
    def dropped_message(res):
        print("[DROP]", res.selector, res.auxiliaries)
    rpc.register_undefined_callback(dropped_message)

    rpc.call(
        "com.apple.instruments.server.services.mobilenotifications",
        'setApplicationStateNotificationsEnabled:', str(True))
    sleep(5)
    rpc.call(
        "com.apple.instruments.server.services.mobilenotifications",
        'setApplicationStateNotificationsEnabled:', str(False))
    # print("aaaaa")
    sleep(10)
    rpc.stop()


if __name__ == '__main__':
    rpc = InstrumentServer().init()
    MobileNotifications(rpc)
    rpc.stop()



"""
dtx massage 协议头
参考 https://github.com/troybowman/dtxmsg
大会 PPT
dtx msg 分为几层，按顺序
1.DTXMessageHeader 头部定义
2.DTXMessagePayloadHeader  DTX Body 数据头定义
3.DTXMessageAux Aux数据
4.CFStringRef 类型数据类型 method 请求的方法声明
"""
import io
import plistlib
from abc import ABC
from enum import Enum

from construct import Struct, Int32ul, Int16ul, Int64ul, Const, Prefixed, GreedyBytes, this, Adapter, Select, \
    GreedyRange, Switch, Default, Int64sl, Int32sl
from ios_device.util.variables import LOG

from . import Log
from .bpylist2 import unarchive, archive
from .exceptions import InstrumentRPCParseError

log = Log.getLogger(LOG.Instrument.value)


class Raw:
    """特殊参数类型判断"""


class RawObj(Raw):
    """ 某些情况 int 类型需要转成 obj
    """

    def __init__(self, *data):
        self.data = data


class RawInt64sl(Raw):
    def __init__(self, *data):
        self.data = data


class RawInt32sl(Raw):
    def __init__(self, *data):
        self.data = data


class RawInt64(Raw):
    def __init__(self, *data):
        self.data = data


class RawInt32(Raw):
    def __init__(self, *data):
        self.data = data


class PlistAdapter(Adapter, ABC):
    def _decode(self, obj, context, path):
        return unarchive(obj)

    def _encode(self, obj, context, path):
        return archive(obj)


dtx_message_header = Struct(
    'magic' / Const(0x1F3D5B79, Int32ul),
    'header_length' / Int32ul,
    'fragment_id' / Default(Int16ul, 0),
    'fragment_count' / Default(Int16ul, 1),
    'payload_length' / Int32ul,
    'identifier' / Int32ul,
    'conversation_index' / Int32ul,
    'channel' / Int32ul,
    'expects_reply' / Int32ul,
)

dtx_message_payload_header = Struct(
    'flags' / Int32ul,
    'aux_length' / Int32ul,
    'total_length' / Int64ul,
)

dtx_message_aux = Struct(
    'magic' / Default(Int64ul, 0x1f0),
    'data' / Prefixed(Int64ul, GreedyRange(Struct(
        'magic' / Select(Const(0xa, Int32ul), Int32ul),
        'type' / Int32ul,
        'value' / Switch(this.type,
                         {1: Prefixed(Int32ul, GreedyBytes), 2: PlistAdapter(Prefixed(Int32ul, GreedyBytes)), 3: Int32sl,
                          4: Int64ul, 5: Int32ul,
                          6: Int64sl},
                         default=GreedyBytes),
    )))
)

server_push_dtx_message_aux = Struct(
    'magic' / Default(Int64ul, 0x1f0),
    'data' / Prefixed(Int64ul, GreedyRange(Struct(
        'type' / Int32ul,
        'value' / Switch(this.type,
                         {1: Prefixed(Int32ul, GreedyBytes), 2: PlistAdapter(Prefixed(Int32ul, GreedyBytes)), 3: Int32sl,
                          4: Int64ul, 5: Int32ul,
                          6: Int64sl},
                         default=GreedyBytes),
    )))
)


class MessageAux:
    def __init__(self):
        self.values = []

    def append_int(self, value: int):
        self.values.append({'type': 3, 'value': value})
        return self

    def append_long(self, value: int):
        self.values.append({'type': 4, 'value': value})
        return self

    def append_signed_int(self, value: int):
        """ 有符号 int """
        self.values.append({'type': 3, 'value': value})
        return self

    def append_signed_long(self, value: int):
        """ 有符号 long """
        self.values.append({'type': 6, 'value': value})
        return self

    def append_obj(self, value):
        self.values.append({'type': 2, 'value': value})
        return self

    def __bytes__(self):
        return dtx_message_aux.build(dict(data=self.values))


class DTXMessage:
    _FLAGS_TYPE = 2

    def __init__(self):
        self._buf = b''
        self._message_header = None
        self._payload_header = None
        self._identifier = None
        self._channel_code = None
        self._expects_reply = None
        self._selector = None
        self._conversation_index = 0
        self._flags = self._FLAGS_TYPE
        self.auxiliaries: MessageAux = MessageAux()

    @classmethod
    def decode(cls, header_data: bytes, payload_data: bytes):
        ret = DTXMessage()
        ret._buf = header_data + payload_data
        payload_io = io.BytesIO(payload_data)
        ret._message_header = dtx_message_header.parse(header_data)
        if not ret._message_header.payload_length > 0:
            return ret

        ret._payload_header = dtx_message_payload_header.parse(payload_io.read(dtx_message_payload_header.sizeof()))
        if ret._payload_header.total_length == 0:
            return ret
        if ret._payload_header.aux_length:
            if ret._payload_header.flags == 0x0:
                auxiliaries = server_push_dtx_message_aux.parse(payload_io.read(ret._payload_header.aux_length)).data
            else:
                auxiliaries = dtx_message_aux.parse(payload_io.read(ret._payload_header.aux_length)).data
            ret.auxiliaries = [i.value for i in auxiliaries]
        else:
            ret.auxiliaries = []

        data = payload_io.read()
        ret._selector = data
        if data:
            for fun in (unarchive, plistlib.loads):  # NSKeyedArchived or Plist
                try:
                    ret._selector = fun(data)
                except:
                    ret._selector = InstrumentRPCParseError(data)
                else:
                    break

        payload_io.close()
        log.debug(f'DTX msg decode: {ret.selector} :{ret.auxiliaries}')
        return ret

    def __bytes__(self) -> bytes:
        aux = bytes(self.auxiliaries) if self.auxiliaries is not None else b''
        if isinstance(self._selector, bytes):
            sel = self._selector
        else:
            sel = archive(self._selector) if self._selector is not None else b''
        payload_header = dtx_message_payload_header.build(
            dict(flags=self._flags, aux_length=len(aux), total_length=len(aux) + len(sel)))
        message_header = dtx_message_header.build(dict(
            header_length=dtx_message_header.sizeof(),
            payload_length=dtx_message_payload_header.sizeof() + len(aux) + len(sel),
            identifier=self._identifier,
            conversation_index=self._conversation_index,
            channel=self._channel_code,
            expects_reply=1 if self._expects_reply else 0
        ))
        payload_buf = message_header + payload_header + aux + sel
        return payload_buf

    @property
    def identifier(self):
        return self._message_header.identifier

    @property
    def channel_code(self):
        return self._message_header.channel

    @property
    def expects_reply(self):
        return self._message_header.expects_reply

    @property
    def selector(self):
        return self._selector

    @property
    def conversation_index(self):
        return self._message_header.conversation_index


def object_to_aux(arg, aux: MessageAux):
    if isinstance(arg, int) and not isinstance(arg, RawObj):
        if abs(arg) < (2 ** 32):
            aux.append_int(arg)
        elif abs(arg) < (2 ** 64):
            aux.append_long(arg)
        else:
            raise ValueError("num too large")
    elif isinstance(arg, Enum):
        arg = arg.value
        aux.append_obj(arg)
    elif isinstance(arg, Raw):
        if isinstance(arg, RawObj):
            for i in arg.data:
                aux.append_obj(i)
        if isinstance(arg, RawInt32sl):
            for i in arg.data:
                aux.append_signed_int(i)
        if isinstance(arg, RawInt64sl):
            for i in arg.data:
                aux.append_signed_long(i)

        if isinstance(arg, RawInt64):
            for i in arg.data:
                aux.append_long(i)

        if isinstance(arg, RawInt32):
            for i in arg.data:
                aux.append_int(i)
    else:
        aux.append_obj(arg)

    return aux


dtx_decode = DTXMessage.decode



import json
from pprint import pprint

import click


class Enconding(json.JSONEncoder):
    def default(self, o):
        if isinstance(o, bytes):
            try:
                return o.decode()
            except:
                return str(o)


def print_json(buf, format=True):
    if format == 'json':
        formatted_json = json.dumps(buf, indent=4, cls=Enconding)
        print(formatted_json)
    elif format == 'flush':
        print(f'\r{buf}', end='', flush=True)
    else:
        print(buf)


class Command(click.Command):

    def __init__(self, *args, **kwargs):
        super(Command, self).__init__(*args, **kwargs)
        self.params[:0] = [
            click.Option(('udid', '--udid'), help='specify unique device identifier'),
            click.Option(('network', '--network'), is_flag=True, help='ios devices on wireless network'),
            click.Option(('format', '--format'),type=click.Choice(['text','json','flush']), help='print format type'),
        ]



"""
获取单个应用 activity数据
"""
import time
import os
import sys


sys.path.append(os.getcwd())
from ios_device.servers.Instrument import  InstrumentServer
from ios_device.util import logging

log = logging.getLogger(__name__)


def activity(rpc, pid):
    def on_callback_message(res):
        print(f"[ACTIVITY] {res.selector}", )
        print("\n")
    rpc.register_channel_callback("com.apple.instruments.server.services.activity", on_callback_message)
    var = rpc.call("com.apple.instruments.server.services.activity", "startSamplingWithPid:", pid).selector
    log.debug(f"start {var}")
    time.sleep(10)
    var = rpc.call("com.apple.instruments.server.services.activity", "stopSampling").selector
    log.debug(f"stop {var}")
    rpc.stop()


if __name__ == '__main__':
    rpc = InstrumentServer().init()
    activity(rpc, 261)
    rpc.stop()



"""
获取系统信息
"""
import os
import sys

sys.path.append(os.getcwd())
from ios_device.servers.Instrument import InstrumentServer
from ios_device.util import logging

log = logging.getLogger(__name__)


def runningProcesses(rpc):
    """ 获取所有运行的进程信息
    :param rpc:
    :return:
    """
    parsed = rpc.call("com.apple.instruments.server.services.deviceinfo", "runningProcesses").selector
    log.debug("runningProcesses:")
    headers = '\t'.join(sorted(parsed[0].keys()))
    log.debug(headers)
    for item in parsed:
        sorted_item = sorted(item.items())
        print('\t'.join(map(str, [v for _, v in sorted_item])))
    rpc.stop()
    return parsed


def execnameForPid(rpc, pid):
    """ 获取 pid 进程的应用名
    :param rpc:
    :param pid:
    :return:
    """
    parsed = rpc.call("com.apple.instruments.server.services.deviceinfo", "execnameForPid:", str(pid)).selector
    print(parsed)
    rpc.stop()
    return parsed


def machTimeInfo(rpc):
    machTimeInfo = rpc.call("com.apple.instruments.server.services.deviceinfo", "machTimeInfo").selector
    print(machTimeInfo)
    rpc.stop()
    return machTimeInfo


def traceCodesFile(rpc):
    parsed = rpc.call("com.apple.instruments.server.services.deviceinfo", "traceCodesFile").selector
    print(parsed)
    rpc.stop()
    return parsed


def networkInformation(rpc):
    parsed = rpc.call("com.apple.instruments.server.services.deviceinfo", "networkInformation").selector
    print(parsed)
    rpc.stop()


def systemInformation(rpc):
    parsed = rpc.call("com.apple.instruments.server.services.deviceinfo", "systemInformation").selector
    print(parsed)
    rpc.stop()


def sysmonProcessAttributes(rpc):
    parsed = rpc.call("com.apple.instruments.server.services.deviceinfo", "sysmonProcessAttributes").selector
    print(parsed)
    rpc.stop()


def sysmonSystemAttributes(rpc):
    parsed = rpc.call("com.apple.instruments.server.services.deviceinfo", "symbolicatorSignaturesForExpiredPids").selector
    print(list(parsed))
    rpc.stop()


if __name__ == '__main__':
    rpc = InstrumentServer().init()
    sysmonProcessAttributes(rpc)
    rpc.stop()



import base64

from datetime import datetime, timedelta

from OpenSSL.crypto import X509, TYPE_RSA, X509Req, PKey, FILETYPE_PEM as PEM, X509Name, X509Extension
from OpenSSL.crypto import load_publickey, dump_privatekey, dump_certificate
from pyasn1.type import univ
from pyasn1.codec.der import encoder as der_encoder, decoder as der_decoder
from cryptography.hazmat.primitives import padding
from cryptography.hazmat.primitives.ciphers import Cipher, algorithms, modes
from cryptography.hazmat.backends import default_backend

__all__ = ['make_certs_and_key']


def make_certs_and_key(device_public_key: bytes):
    priv_key = PKey()
    priv_key.generate_key(TYPE_RSA, 2048)

    req = make_req(priv_key)
    cert = make_cert(req, priv_key)

    dev_key = load_publickey(PEM, convert_PKCS1_to_PKCS8_pubkey(device_public_key))
    dev_key._only_public = False
    dev_req = make_req(dev_key, 'Device')
    dev_cert = make_cert(dev_req, priv_key)

    return dump_certificate(PEM, cert), dump_privatekey(PEM, priv_key), dump_certificate(PEM, dev_cert)


def convert_PKCS1_to_PKCS8_pubkey(data: bytes) -> bytes:
    pubkey_pkcs1_b64 = b''.join(data.split(b'\n')[1:-2])
    pubkey_pkcs1, restOfInput = der_decoder.decode(base64.b64decode(pubkey_pkcs1_b64))
    bit_str = univ.Sequence()
    bit_str.setComponentByPosition(0, univ.Integer(pubkey_pkcs1[0]))
    bit_str.setComponentByPosition(1, univ.Integer(pubkey_pkcs1[1]))
    bit_str = der_encoder.encode(bit_str)
    try:
        bit_str = ''.join([('00000000' + bin(ord(x))[2:])[-8:] for x in list(bit_str)])
    except Exception:
        bit_str = ''.join([('00000000' + bin(x)[2:])[-8:] for x in list(bit_str)])
    bit_str = univ.BitString("'%s'B" % bit_str)
    pubkeyid = univ.Sequence()
    pubkeyid.setComponentByPosition(0, univ.ObjectIdentifier('1.2.840.113549.1.1.1'))  # == OID for rsaEncryption
    pubkeyid.setComponentByPosition(1, univ.Null(''))
    pubkey_seq = univ.Sequence()
    pubkey_seq.setComponentByPosition(0, pubkeyid)
    pubkey_seq.setComponentByPosition(1, bit_str)
    pubkey = der_encoder.encode(pubkey_seq)
    return b'-----BEGIN PUBLIC KEY-----\n' + base64.encodebytes(pubkey) + b'-----END PUBLIC KEY-----\n'


def x509_time(**kwargs) -> bytes:
    dt = datetime.utcnow() + timedelta(**kwargs)
    return dt.strftime('%Y%m%d%H%M%SZ').encode('utf-8')


def make_cert(req: X509Req, ca_pkey: PKey) -> X509:
    cert = X509()
    cert.set_serial_number(1)
    cert.set_version(2)
    cert.set_subject(req.get_subject())
    cert.set_pubkey(req.get_pubkey())
    cert.set_notBefore(x509_time(minutes=-1))
    cert.set_notAfter(x509_time(days=30))
    ca_subj = cert.get_subject()
    ca_subj.CN = 'The YueChen'
    ca_subj.O = 'The Organization Otherwise Known as YueChen CA, Inc.'
    cert.set_issuer(ca_subj)
    cert.sign(ca_pkey, 'sha1')
    return cert


def make_req(pub_key, cn=None, digest=None) -> X509Req:
    req = X509Req()
    req.set_version(2)
    req.set_pubkey(pub_key)
    if cn is not None:
        subject = req.get_subject()
        subject.CN = cn.encode('utf-8')
    if digest:
        req.sign(pub_key, digest)
    return req


class AESCrypto(object):
    AES_CBC_IV = b'\x00' * 16

    @classmethod
    def cbc_encrypt(cls, data, key):
        if not isinstance(data, bytes):
            data = data.encode()

        cipher = Cipher(algorithms.AES(key),
                        modes.CBC(cls.AES_CBC_IV),
                        backend=default_backend())
        encryptor = cipher.encryptor()

        padded_data = encryptor.update(cls.pkcs7_padding(data))

        return padded_data

    @classmethod
    def cbc_decrypt(cls, data, key):
        if not isinstance(data, bytes):
            data = data.encode()

        cipher = Cipher(algorithms.AES(key),
                        modes.CBC(cls.AES_CBC_IV),
                        backend=default_backend())
        decryptor = cipher.decryptor()

        uppaded_data = cls.pkcs7_unpadding(decryptor.update(data))
        return uppaded_data

    @staticmethod
    def pkcs7_unpadding(padded_data):
        unpadder = padding.PKCS7(algorithms.AES.block_size).unpadder()
        data = unpadder.update(padded_data)

        try:
            uppadded_data = data + unpadder.finalize()
        except ValueError:
            raise Exception('无效的加密信息!')
        else:
            return uppadded_data

    @staticmethod
    def pkcs7_padding(data):
        if not isinstance(data, bytes):
            data = data.encode()

        padder = padding.PKCS7(algorithms.AES.block_size).padder()

        padded_data = padder.update(data) + padder.finalize()

        return padded_data



"""
@Date    : 2020-12-18
@Author  : liyachao
"""
from setuptools import setup, find_packages
from ios_device import __version__

REQUIREMENTS = [i.strip() for i in open("requirements.txt").readlines()]


setup(name='py_ios_device',
      version=__version__,
      description='Get ios data and operate ios devices',
      author='chenpeijie',
      author_email='cpjsf@163.com',
      maintainer='chenpeijie',
      maintainer_email='',
      url='https://github.com/YueChen-C/py-ios-device',
      packages=find_packages(),
      long_description=open('README.md').read(),
      long_description_content_type="text/markdown",
      install_requires=REQUIREMENTS,
      python_requires=">=3.7",
      include_package_data=True,
      classifiers=[
          "Programming Language :: Python :: 3",
          "Operating System :: OS Independent",
      ],
      entry_points={
          'console_scripts':{
              'pyidevice=ios_device.main:cli'
          }
      },
)



from errno import ENOENT, ENOTDIR

from .constants import AFC_E_UNKNOWN_ERROR, AFC_ERROR_NAMES, AFC_E_OBJECT_NOT_FOUND, AFC_E_OBJECT_IS_DIR

AFC_TO_OS_ERROR_CODES = {
    AFC_E_OBJECT_NOT_FOUND: ENOENT,
    AFC_E_OBJECT_IS_DIR: ENOTDIR,
}


class PyPodException(Exception):
    """Base exception class for all exceptions in PyPod"""


class LockdownException(PyPodException):
    pass


class PairingError(LockdownException):
    pass


class NotTrustedError(PairingError):
    pass


class FatalPairingError(PairingError):
    pass


class NotPairedError(LockdownException):
    pass


class CannotStopSessionError(LockdownException):
    pass


class StartServiceError(LockdownException):
    pass


class InitializationError(LockdownException):
    pass


class MuxError(PyPodException):
    pass


class MuxVersionError(MuxError):
    pass


class NoMuxDeviceFound(MuxError):
    pass


class iOSError(PyPodException, OSError):
    """Generic exception for AFC errors or errors that would normally be raised by the OS"""

    def __init__(self, errno=None, afc_errno=AFC_E_UNKNOWN_ERROR, *args, **kwargs):
        errno = AFC_TO_OS_ERROR_CODES.get(afc_errno) if errno is None else errno
        # noinspection PyArgumentList
        super().__init__(errno, *args, **kwargs)
        self.afc_errno = afc_errno

    def __str__(self):
        name = AFC_ERROR_NAMES.get(self.afc_errno, 'UNKNOWN ERROR')
        return f'{self.__class__.__name__}[afc={self.afc_errno}/{name}][os={self.errno}] {self.strerror}'


class iFileNotFoundError(PyPodException, FileNotFoundError):
    def __init__(self, *args, **kwargs):
        # noinspection PyArgumentList
        super().__init__(ENOENT, *args, **kwargs)


class iDeviceFileClosed(iOSError):
    pass


class InstrumentRPCParseError:
    def __init__(self, data=None):
        self.data = data



import os
import sys
import time
import uuid

from ios_device.cli.base import InstrumentDeviceInfo
from ios_device.servers.Instrument import InstrumentServer
from ios_device.util.exceptions import InstrumentRPCParseError
from ios_device.util.kperf_data import KdBufParser, KperfData

sys.path.append(os.getcwd())

NANO_SECOND = 1e9  # ns
MOVIE_FRAME_COST = 1 / 24


def graphics_display(rpc):

    def on_graphics_message(res):
        if type(res.selector) is InstrumentRPCParseError:
            for args in Kperf.to_str(res.selector.data):
                print(args)

    rpc.register_channel_callback("com.apple.instruments.server.services.coreprofilesessiontap", on_graphics_message)
    traceCodesFile = InstrumentDeviceInfo(rpc).traceCodesFile()
    Kperf = KperfData(traceCodesFile)
    rpc.call("com.apple.instruments.server.services.coreprofilesessiontap", "setConfig:",
             {
                 'tc': [{
                     'csd': 128,
                     'kdf2': {0xffffffff},
                     'ta': [[3], [0], [2], [1, 1, 0]],
                     'tk': 3,
                     'uuid': str(uuid.uuid4()),
                 }],
                 'rp': 100,
                 'bm': 0,
             })
    rpc.call("com.apple.instruments.server.services.coreprofilesessiontap", "start")
    time.sleep(10000)
    rpc.call("com.apple.instruments.server.services.coreprofilesessiontap", "stop")
    rpc.stop()


if __name__ == '__main__':
    rpc = InstrumentServer().init()
    graphics_display(rpc)
    rpc.stop()



import json
import os
import sys

sys.path.append(os.getcwd())
from ios_device.servers.Instrument import InstrumentServer
# from ios_device.util import logging

# log = logging.getLogger(__name__)


def channels(rpc):
    rpc.init()
    print(rpc._published_capabilities)

if __name__ == '__main__':
    rpc = InstrumentServer()
    channels(rpc)
    rpc.stop()
